//+=============================================================================
//
// file :         PLCaccess.cpp
//
// description :  C++ source for the PLCaccess and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                PLCaccess are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: graziano $
//
// $Revision: 1.35 $
//
// $Log: PLCaccess.cpp,v $
// Revision 1.35  2018-01-18 14:01:52  graziano
// fixed Booster_board_disabled_142
//
// Revision 1.34  2018-01-18 13:40:01  graziano
// fixed
//
// Revision 1.32  2018-01-17 10:30:12  graziano
// fixed ronda_rbl[9] -> db51 302.0
//
// Revision 1.31  2017-08-21 12:54:37  graziano
// added TopUp_GeV, TopUp_Min_I and booster_board_disabled_xxx attributes
//
// Revision 1.30  2017-08-11 14:59:27  graziano
// added log of db51 with sr and fermi pss format,
// fixed timestamp
//
// Revision 1.29  2017-03-29 14:36:50  claudio
// ticket 1934: compile for tango-9
//
// Revision 1.28  2016-04-29 08:50:56  graziano
// fixed bug
//
// Revision 1.27  2016-04-06 11:54:39  graziano
// fixed dst
//
// Revision 1.26  2016-01-15 11:21:02  graziano
// added db state and error
//
// Revision 1.25  2015-09-15 12:52:15  graziano
// fixed bug in check_guntr_abi
//
// Revision 1.24  2014-11-13 09:59:56  graziano
// tango 8.1.2.c and Device_4Impl
//
// Revision 1.23  2013-01-07 10:31:30  graziano
// fixed del_posto_bo and del_posto_inj to send maximum 1 bit valorized as 0
//
// Revision 1.22  2013-01-04 11:01:24  graziano
// added release
//
// Revision 1.21  2013-01-04 10:38:20  graziano
// fixed log of people
//
// Revision 1.20  2012-07-05 08:03:12  zambonl
// .ndd
//
// Revision 1.19  2010/05/17 08:08:16  zambonl
// ndd
//
// Revision 1.18  2010/01/21 12:27:32  zambonl
// ndd
//
// Revision 1.17  2010/01/12 10:15:26  zambonl
// ndd
//
// Revision 1.16  2010/01/12 09:20:37  zambonl
// ndd
//
// Revision 1.15  2009/11/27 15:07:14  sara
// fixed bug in plcsrvthread, wrong msec in timestamp
//
// Revision 1.14  2009/11/25 15:41:45  sara
// added reset topup error command
//
// Revision 1.13  2009/11/18 16:53:36  sara
// topup version
//
// Revision 1.12  2008/10/13 12:26:36  sara
// *** empty log message ***
//
// Revision 1.11  2008/10/13 08:32:02  sara
// safe message recognition, adjustments in ctrl_names and ronda_names
// attributes
//
// Revision 1.10  2008/07/01 13:39:26  sara
// added PLC clock syncronizing command
//
// Revision 1.9  2008/03/17 14:29:56  sara
// Includes access logging and a new attribute: Ramping_condition
//
// Revision 1.8  2008/01/16 16:16:50  sara
// added beam stopper 2.1 signals, changes in rf_stat attribute
//
// Revision 1.7  2007/11/22 16:49:14  sara
// new attributes:Gun_trigger_on, Gun_trigger_interlock_abi, Gun_trigger_access_abi
// enabled database log
//
// Revision 1.6  2007/11/05 10:37:33  sara
// added commands gun_on and gun_off, modified and added signals
//
// Revision 1.5  2007/10/08 14:40:21  sara
// changes in ronda booster fail cause
//
// Revision 1.4  2007/10/05 09:14:36  sara
// added attribute "version"
//
// Revision 1.3  2007/08/31 16:45:44  sara
// *** empty log message ***
//
// Revision 1.2  2007/08/13 10:26:52  sara
// timing thread for receive from PLC timeout
//
// Revision 1.1.1.1  2007/07/02 09:29:34  sara
// Initial release
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name        |  Method's name
//	----------------------------------------
//  State                 |  dev_state()
//  Status                |  dev_status()
//  Open_bst_b13_1        |  open_bst_b13_1()
//  Close_bst_b13_1       |  close_bst_b13_1()
//  Open_bst_bts1_1       |  open_bst_bts1_1()
//  Close_bst_bts1_1      |  close_bst_bts1_1()
//  Open_bst_bts1_2       |  open_bst_bts1_2()
//  Close_bst_bts1_2      |  close_bst_bts1_2()
//  Open_bst_bts1_3       |  open_bst_bts1_3()
//  Close_bst_bts1_3      |  close_bst_bts1_3()
//  Del_posto_bo          |  del_posto_bo()
//  Reint_pasv            |  reint_pasv()
//  Close_bst_bl          |  close_bst_bl()
//  Set_top_up            |  set_top_up()
//  Ack_eme               |  ack_eme()
//  Call_ronda_bo         |  call_ronda_bo()
//  Start_ronda_bo        |  start_ronda_bo()
//  End_ronda_bo          |  end_ronda_bo()
//  Start_ronda_int       |  start_ronda_int()
//  End_ronda_int         |  end_ronda_int()
//  Start_ronda_dx        |  start_ronda_dx()
//  End_ronda_dx          |  end_ronda_dx()
//  Start_ronda_sx        |  start_ronda_sx()
//  End_ronda_sx          |  end_ronda_sx()
//  Start_ronda_roof      |  start_ronda_roof()
//  End_ronda_roof        |  end_ronda_roof()
//  Ripr_in_boost         |  ripr_in_boost()
//  Ripr_in_inj           |  ripr_in_inj()
//  Open_door_bo          |  open_door_bo()
//  Open_door_inj         |  open_door_inj()
//  Del_posto_inj         |  del_posto_inj()
//  Confirm_open_13_1     |  confirm_open_13_1()
//  Confirm_close_13_1    |  confirm_close_13_1()
//  Confirm_open_1_1      |  confirm_open_1_1()
//  Confirm_close_1_1     |  confirm_close_1_1()
//  Confirm_open_1_2      |  confirm_open_1_2()
//  Confirm_close_1_2     |  confirm_close_1_2()
//  Confirm_open_1_3      |  confirm_open_1_3()
//  Confirm_close_1_3     |  confirm_close_1_3()
//  Confirm_reint_pasv    |  confirm_reint_pasv()
//  Confirm_close_bst_bl  |  confirm_close_bst_bl()
//  Confirm_set_topup     |  confirm_set_topup()
//  Confirm_ack_eme       |  confirm_ack_eme()
//  Echo_db50             |  echo_db50()
//  Echo_db51             |  echo_db51()
//  Echo_db121            |  echo_db121()
//  Echo_db122            |  echo_db122()
//  Gun_on                |  gun_on()
//  Gun_off               |  gun_off()
//  Syncronize            |  syncronize()
//  Rst_top_up            |  rst_top_up()
//  Err_topup_rst         |  err_topup_rst()
//
//===================================================================


#include <tango.h>
#include <PLCaccess.h>
#include <PLCaccessClass.h>
#include <iomanip>	//used for debug dump of db
#include "plcsrvthread.h"
#include "plctimingthread.h"

#ifndef SERVER_BUILDTIME
#define SERVER_BUILDTIME   __DATE__ " "  __TIME__
#endif

const char version_string[] = "$Build: " SERVER_BUILDTIME " $";

namespace PLCaccess_ns
{
static u_short posti[] = {
		canc0,
		canc1,
		canc2,
		canc3,
		canc4,
		canc5,
		canc6,
		canc7
};

static int check_topup_inj_byte[CHECK_TOPUP_INJ_LEN] ={68,68,64,64,64,107,0,132,  27,27,27,27,27,  0,7,68};
//[8]->27-11->DB51.DBX67.3, [9]->27-13->DB51.DBX67.5, [10]->27-8->DB51.DBX67.0, [11]->27-12->DB51.DBX67.4, [12]->27-10->DB51.DBX67.2
static int check_topup_inj_bit[CHECK_TOPUP_INJ_LEN] = { 1, 2, 0, 1,10,  1,0,  1,  11,13, 8,12,10,  4,5,0};
static int check_topup_test_byte[CHECK_TOPUP_TEST_LEN] ={68,64,64,64,107,0,0,0,1,1,1,132,  27,  7};
//[12]->27-8->DB51.DBX67.0
static int check_topup_test_bit[CHECK_TOPUP_TEST_LEN] = { 1, 1, 0,10,  1,0,0,8,0,8,0,  0,   8,  6};
static int check_inj_sr_byte[CHECK_INJ_SR_LEN] ={68,64,64,64,107,0,0,0,1,1,1,7,7,7};
static int check_inj_sr_bit[CHECK_INJ_SR_LEN] ={1,1,0,10,1,0,0,8,0,8,8,5,6,4};
static int check_open_bst1_1_byte[CHECK_OPN_BST1_1_LEN] ={7,1,76,70,74,74,74,76,7,7,76,76,0};
static int check_open_bst1_1_bit[CHECK_OPN_BST1_1_LEN] ={0,8,2,2,3,7,7,6,6,5,6,2,15};
static int check_open_bst1_2_byte[CHECK_OPN_BST1_2_LEN] ={7,1,76,70,74,74,74,76,7,7,76,76,1};
static int check_open_bst1_2_bit[CHECK_OPN_BST1_2_LEN] ={0,8,2,2,3,7,7,6,6,5,6,2,7};
static int check_open_bst1_3_byte[CHECK_OPN_BST1_3_LEN] ={0,70,70,70,74,74,74,76,7,7,7,70,1};
static int check_open_bst1_3_bit[CHECK_OPN_BST1_3_LEN] ={1,3,3,2,3,7,7,6,6,5,5,2,15};
static int check_guntr_abi_byte[CHECK_GUN_TR_ABI_LEN] ={68,32,32,32,32,32,72,0,55,55,55,0,  27,  70,76,74,74,  27,  7,7,  27,27,27,27,27,27,27,  7,7, 6};
//[12]->27-8->DB51.DBX67.0, [17]->27-8->DB51.DBX67.0, [20]->27-13->DB51.DBX67.5, [21]->27-9->DB51.DBX67.1, [22]->27-10->DB51.DBX67.2, [23]->27-12->DB51.DBX67.4, [24]->27-14->DB51.DBX67.6, [25]->27-8->DB51.DBX67.0, [26]->27-15->DB51.DBX67.7
static int check_guntr_abi_bit[CHECK_GUN_TR_ABI_LEN] = { 1, 3, 0, 4, 1, 2, 9,0, 5, 6, 4,9,   8,   2, 6, 3, 7,   8,  6,5,  13, 9,10,12,14, 8,15,  5,6,15};
static int topup_err_byte[TOPUP_ERR_LEN] ={68,68,132,  27,27,27,27,27,27,27,  68,  27,  8,  27,27};
//[3]->27-12->DB51.DBX67.4, [4]->27-11->DB51.DBX67.3, [5]->27-13->DB51.DBX67.5, [6]->27-8->DB51.DBX67.0, [7]->27-15->DB51.DBX67.7, [8]->27-9->DB51.DBX67.1, [9]->27-10->DB51.DBX67.2, [11]->27-14->DB51.DBX67.6, [13]->27-5->DB51.DBX66.5, [14]->27-6->DB51.DBX67.6
static int topup_err_bit[TOPUP_ERR_LEN] = {1,  2,  0,  12,11,13,8, 15,9, 10,   1,  14, 12,   5, 6};

}

namespace PLCaccess_ns
{

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::PLCaccess(string &s)
// 
// description : 	constructor for simulated PLCaccess
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
PLCaccess::PLCaccess(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

PLCaccess::PLCaccess(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

PLCaccess::PLCaccess(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void PLCaccess::delete_device()
{
	//	Delete device's allocated object
	int t_finish = 3000;// h_timeout[0]time in msec to wait for the thread to complete
	abortflag = 1; //stops thread
	int secs = (int)t_finish/1000;
	int nanos = (t_finish*1000-secs)*1000000000;
	omni_thread::sleep(secs,nanos);// for the last thread to finish
	usleep(11000000);
	delete badgeackbo;
	delete badgeackinj;
	CORBA::string_free(plc_timestamp);
	for(int i=0;i<DB_NAMES_LEN;i++){
		delete ctrlacc_names[i];
		delete rda_names[i];
	}
	delete [] ctrlacc_names;
	delete [] rda_names;
	for(int i=0;i<NUM_POSTO;i++){
		delete badgeinj[i];
		delete badgebo[i];
	}
//	delete [] badgeinj;
//	delete [] badgebo;
	delete attr_DBState_read;
	delete [] attr_DBError_read;
	delete [] attr_Min_I_TopUp_read;
	delete [] attr_Topup_GeV_read;
	
	delete [] attr_Booster_board_disabled_008_read;
	delete [] attr_Booster_board_disabled_048_read;
	delete [] attr_Booster_board_disabled_064_read;
	delete [] attr_Booster_board_disabled_080_read;
	delete [] attr_Booster_board_disabled_100_read;
	delete [] attr_Booster_board_disabled_112_read;
	delete [] attr_Booster_board_disabled_118_read;
	delete [] attr_Booster_board_disabled_124_read;
	delete [] attr_Booster_board_disabled_130_read;
	delete [] attr_Booster_board_disabled_136_read;
	delete [] attr_Booster_board_disabled_142_read;
	delete [] attr_Booster_board_disabled_148_read;
	delete [] attr_Booster_board_disabled_223_read;
	delete [] attr_Booster_board_disabled_229_read;
	delete [] attr_Booster_board_disabled_235_read;
	delete [] attr_Booster_board_disabled_247_read;
	delete [] attr_Booster_board_disabled_253_read;
	delete [] attr_Booster_board_disabled_259_read;
	delete [] attr_Booster_board_disabled_265_read;
	delete [] attr_Booster_board_disabled_278_read;
	delete [] attr_Booster_board_disabled_291_read;
	delete [] attr_Booster_board_disabled_304_read;
	delete [] attr_Booster_board_disabled_310_read;
	delete [] attr_Booster_board_disabled_316_read;
	delete [] attr_Booster_board_disabled_321_read;
	delete [] attr_Booster_board_disabled_333_read;
	delete [] attr_Booster_board_disabled_339_read;
	delete [] attr_Booster_board_disabled_345_read;

	delete [] attr_BTSReal_read;
	delete [] attr_BTSBool_read;
	delete [] attr_I_BTS_Thresh_2Gev_read;
	delete [] attr_I_BTS_Thresh_2_4Gev_read;
	delete [] attr_I_BTS_read;
	delete [] attr_I_MAX_Lost_read;
	
	if(sock)
		close(sock);
	if(new_sd)
		close(new_sd);
	//if(fdlog)
	//	close(fdlog);
	delete mutex;
	delete [] sw_version;
  	mysql_close(&dbp);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void PLCaccess::init_device()
{
	INFO_STREAM << "PLCaccess::PLCaccess() create device " << device_name << " ver.:" << version_string << endl;

	// Initialise variables to default values
	//--------------------------------------------
	memset(blk_db51_buffer, 0, NUM_DATA_BLK*2);
	strcpy(dbtable,"booster");	//TODO: get from properties
	
	abortflag = 0;
	srvloop = 0;
	sock = 0;
	new_sd = 0;
	for(int i=0;i<5;i++)ronda_req_resettable[i] = false;	
	u_short mybuf[sizeof(db_51)/2];
	logfile2 = "/ELETTRA_log/logs/plcaccess-srv.output.txt";
	echoreq = 0;//no echo request
	get_device_property();
	char buf3[BUF_LEN];
        memset (buf3,0,sizeof(buf3));
	rondanames_ok = false;//not updated yet
	ctrlnames_ok = false;//not updated yet

	sw_version = new char[BUF_LEN];


	memset(blk_db50_buffer, 0, 18*sizeof(pers_name));

	//find hostname
	char indi[INET_ADDRSTRLEN];
	struct hostent* he;
	string nome = Tango::Util::instance()->get_host_name();
	
        he = gethostbyname(nome.c_str());
        if(he == NULL){
		DEBUG_STREAM << "PLCaccess::Error getting host names" << endl;
        }
	int i = 0;
        while(he->h_addr_list[i] != NULL){
                if(inet_ntop(AF_INET, he->h_addr_list[i], indi, INET_ADDRSTRLEN) <= 0){
                        perror("inet_ntop");
                }
		DEBUG_STREAM << "PLCaccess::found host name: "<<  indi << endl;
		i++;
	}
	srv_host = indi;
	
	h_timeout[0] = 11000;
	h_timeout[0] = atoi(hbeat.c_str());
	
	badgeackbo = new char[NUM_BIT+2];
	badgeackinj = new char[NUM_BIT+2];
	for(int i=0;i<NUM_BIT+2;i++){
		badgeackbo[i] = ' ';
		badgeackinj[i] = ' ';
	}
	plc_timestamp = CORBA::string_alloc(TSTAMP_SIZE);


	ctrlacc_names = new Tango::DevString[DB_NAMES_LEN];
	for(int i=0;i<DB_NAMES_LEN;i++){
		ctrlacc_names[i] = new char[NUM_BIT+1];
		memset(ctrlacc_names[i],0,NUM_BIT+1);
		strcpy(ctrlacc_names[i], " ");
	}
	rda_names = new Tango::DevString[DB_NAMES_LEN];
	for(int i=0;i<DB_NAMES_LEN;i++){
		rda_names[i] = new char[NUM_BIT+1];
		memset(rda_names[i],0,NUM_BIT+1);
		strcpy(rda_names[i], " ");
	}
//	badgebo = new Tango::DevString[NUM_POSTO];
	for(int i=0;i<NUM_POSTO;i++){
		badgebo[i] = new char[NUM_BIT+1];
		memset(badgebo[i],0,NUM_BIT+1);
		//strcpy(badgebo[i], " ");
	}
//	badgeinj = new Tango::DevString[NUM_POSTO];
	for(int i=0;i<NUM_POSTO;i++){
		badgeinj[i] = new char[NUM_BIT+1];
		memset(badgeinj[i],0,NUM_BIT+1);
		strcpy(badgeinj[i], " ");
	}
	gun_trigger_on = gun_trigger_interlock_abi = gun_trigger_access_abi = false;
	
	if(sock > 0)
		close(sock);
	memset (&message,0,(sizeof(PLCaccess_ns::db_50)));
	memset (&message2,0,(sizeof(PLCaccess_ns::db_51)));
	memset (&cmd_msg,0,(sizeof(PLCaccess_ns::plc_tx)));
	memset (&tst50,0,(sizeof(PLCaccess_ns::tstamp_s)));	
	memset (&old_tst50,0,(sizeof(PLCaccess_ns::tstamp_s)));	
	memset (&tst51,0,(sizeof(PLCaccess_ns::tstamp_s)));
	memset (&ctrl_names,0,(sizeof(PLCaccess_ns::db_12x)));
	memset (&ronda_names,0,(sizeof(PLCaccess_ns::db_12x)));
	
	attr_DBState_read = new Tango::DevBoolean();
	*attr_DBState_read = false;
	attr_DBError_read = new Tango::DevString[1];
	strcpy(db_error,"");
	attr_DBError_read[0] = (char *)db_error;
	
	attr_Min_I_TopUp_read = new Tango::DevLong[2];
	attr_Topup_GeV_read = new Tango::DevBoolean[2];
	
	attr_Booster_board_disabled_008_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_048_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_064_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_080_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_100_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_112_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_118_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_124_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_130_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_136_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_142_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_148_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_223_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_229_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_235_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_247_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_253_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_259_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_265_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_278_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_291_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_304_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_310_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_316_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_321_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_333_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_339_read = new Tango::DevBoolean[16];
	attr_Booster_board_disabled_345_read = new Tango::DevBoolean[16];

	attr_BTSReal_read = new Tango::DevDouble[NUM_FLOAT_BLK];
	attr_BTSBool_read = new Tango::DevBoolean[NUM_BOOL_BLK*8];
	attr_I_BTS_Thresh_2Gev_read = new Tango::DevDouble[NUM_FLOAT_BLK];
	attr_I_BTS_Thresh_2_4Gev_read = new Tango::DevLong[NUM_DWORD_BLK];
	attr_I_BTS_read = new Tango::DevDouble[NUM_FLOAT2_BLK-2];
	attr_I_MAX_Lost_read = new Tango::DevDouble[2];

	DEBUG_STREAM <<"size of topup_dbcurr structure: "<<sizeof(PLCaccess_ns::topup_dbcurr)<<endl;
	DEBUG_STREAM <<"size of nshot structure: "<<sizeof(PLCaccess_ns::nshot)<<endl;
	DEBUG_STREAM <<"size of tblk structure: "<<sizeof(PLCaccess_ns::tblk)<<endl;
//TOPUP
	ontopup = false;
	topup_sem = 0;//semaphore
	shot_number = 0;
	for(int k=0;k<DB181_BIT_LEN;k++){
		msg_all[k] = false;
	}
	for(int k=0;k<CURR_BUF_XLEN;k++){
		c0[k] = 0;
		c1[k] = 0;
		c2[k] = 0;
		c3[k] = 0;
		c4[k] = 0;
		c5[k] = 0;
		c6[k] = 0;
		c7[k] = 0;
		r8[k] = 0;
		d0[k] = 0;
	}
	for(int k=0;k<TOPUP_BLOCK_LEN;k++)c_meas[k] = 0;
	curr_lost_short_term = 0;
	curr_lost_long_term = 0;
	cmd_on_topup = false;
	cmd_rst_err = false;
//TOPUP
	mutex = new omni_mutex();
	srvloop = new plcsrvthread(this);
	srvloop->start();
	DEBUG_STREAM << "PLCaccess::init_device() listening thread started" << endl;
	
			
	timeloop = new plctimingthread(this);
	//timeloop->StartUndetached();
	timeloop->start();
	DEBUG_STREAM << "PLCaccess::timing thread started" << endl;
	
	memset(&mybuf, 0, sizeof(db_51));
	memset(&buf_db_51, 0, sizeof(db_51));
	u_short num = 0xffee;
	int offset = 6;
	mybuf[offset]= num;
	bcopy(&mybuf, &buf_db_51, sizeof(db_51));
	
	//get mysql connection
	if(!mysql_init(&dbp)){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("init db log error");
		DEBUG_STREAM << "PLCaccess::init_device() init db log error" << endl;
		*attr_DBState_read = false;
		strcpy(db_error, "init db log error");
		try
		{
			push_change_event("DBState",attr_DBState_read);
			push_archive_event("DBState",attr_DBState_read);
		} catch(Tango::DevFailed &e)
		{
			DEBUG_STREAM << "PLCaccess::init_device() error pushing events, err=" << e.errors[0].desc << endl;
		}
	}
	else
		DEBUG_STREAM << "PLCaccess::init_device() init db log" << endl;
	my_bool my_auto_reconnect=1;
	if(mysql_options(&dbp,MYSQL_OPT_RECONNECT,&my_auto_reconnect) !=0){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("mysql auto reconnection error");
		DEBUG_STREAM << "PLCaccess::init_device() mysql auto reconnection error" << mysql_error(&dbp) << endl;
		*attr_DBState_read = false;
		strcpy(db_error, "MYSQL_OPT_RECONNECT error");
		try
		{
			push_change_event("DBState",attr_DBState_read);
			push_archive_event("DBState",attr_DBState_read);
		} catch(Tango::DevFailed &e)
		{
			DEBUG_STREAM << "PLCaccess::init_device() error pushing events, err=" << e.errors[0].desc << endl;
		}
	}
	else
		DEBUG_STREAM << "PLCaccess::init_device() mysql auto reconnection OK" << endl;
	string dbhost;
	string dbuser;
	string dbpw;
	string dbname;
	string dbport;
	int dbportint;
	vector<string>::iterator iter = db_param.begin();
	dbhost = ((string)*iter);
	iter++;
	dbuser = ((string)*iter);
	iter++;
	dbpw = ((string)*iter);
	iter++;
	dbname = ((string)*iter);
	iter++;
	dbport = ((string)*iter);
	dbportint = atoi(dbport.c_str());
	if(dbportint!=0){
		if(!mysql_real_connect(&dbp, dbhost.c_str(), dbuser.c_str(), dbpw.c_str(), dbname.c_str(), dbportint, NULL, 0)){
			if(get_state()!=Tango::FAULT)
				set_state(Tango::DISABLE);
			set_status("db log connection error");
			DEBUG_STREAM << "PLCaccess::init_device() db log connection error" << endl;
			*attr_DBState_read = false;
			strcpy(db_error, "mysql_real_connect error");
			try
			{
				push_change_event("DBState",attr_DBState_read);
				push_archive_event("DBState",attr_DBState_read);
			} catch(Tango::DevFailed &e)
			{
				DEBUG_STREAM << "PLCaccess::init_device() error pushing events, err=" << e.errors[0].desc << endl;
			}
		}
		else{
			if(get_state()==Tango::DISABLE)
				set_state(Tango::ON);
			set_status("db log connected");
			DEBUG_STREAM << "PLCaccess::init_device() db log connection OK" << endl;
			*attr_DBState_read = true;
			try
			{
				push_change_event("DBState",attr_DBState_read);
				push_archive_event("DBState",attr_DBState_read);
			} catch(Tango::DevFailed &e)
			{
				DEBUG_STREAM << "PLCaccess::init_device() error pushing events, err=" << e.errors[0].desc << endl;
			}
		}
	}
	else{
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("db log connection error");
		DEBUG_STREAM << "PLCaccess::init_device() db log connection error" << endl;
	}


/*	if((fdlog = open(logfile2, O_RDWR | O_APPEND , S_IRWXU | S_IRWXG | S_IRWXO)) < 0){
		DEBUG_STREAM << "PLCaccess::init_device() cannot open log file:" << strerror(errno) << endl;
		if(get_state()!=Tango::FAULT)
			set_state(Tango::UNKNOWN);
		set_status("logging file opening error");
	}
	else{
		int writefile = 0;
		int ln = 0;
		string pie;
		sprintf(&pie[0],"data structures(bytes): db50 %d - db51 %d - dbcmd %d - db12x %d\n",sizeof(db_50),sizeof(db_51),sizeof(plc_tx),sizeof(db_12x));
		for(int j=0;j<strlen(pie.c_str());j++){
			buf3[ln+j] = pie.c_str()[j];
		}
		ln+=strlen(pie.c_str());
		buf3[ln] = '\n';
		if((writefile = write (fdlog, buf3, ln+1)<0)){
			DEBUG_STREAM << "PLCaccess::init_device() write head file failure:" << strerror(errno) << endl;
		} 
	
	}
*///topup

//attribute init
	for(int j=0;j<24;j++){
		old_doors_all[j] = false;
	}
	for(int j=0;j<24;j++){//at the beginning of ronda, doors are closed
		doors_ronda_all[j] = true;
	}
	for(int j=0;j<5;j++){//clear all ronda req causes
		ronda_req_cause[j] = -1;
	}
	for(int j=0;j<4;j++){//clear all memo barr
		memo_barr[j] = false;
	}
	for(int j=0;j<21;j++){
		ronda_bo_fail_cause[j] = false;
	}
	for(int j=0;j<8;j++){
		ronda_int_fail_cause[j] = false;
	}
	for(int j=0;j<7;j++){
		ronda_roof_fail_cause[j] = false;
	}
	for(int j=0;j<9;j++){
		ronda_dx_fail_cause[j] = false;
	}
	for(int j=0;j<12;j++){
		ronda_sx_fail_cause[j] = false;
	}
	memo_err_inout[0] = false;
	memo_err_inout[1] = false;
	
	// assign pointers for read attributes
	attr_Topup_err_read = &topup_err[0];
	attr_Curr_lost_short_term_read = &curr_lost_short_term;
	attr_Curr_lost_long_term_read = &curr_lost_long_term;
	attr_LOST_LT_current_read = &(c7[0]);
	attr_Check_topup_inj_read = &check_topup_inj[0];
	attr_Check_topup_test_read = &check_topup_test[0];
	attr_Check_inj_sr_read = &check_inj_sr[0];
	attr_Check_open_bst1_1_read = &check_open_bst1_1[0];
	attr_Check_open_bst1_2_read = &check_open_bst1_2[0];
	attr_Check_open_bst1_3_read = &check_open_bst1_3[0];
	attr_Check_guntr_abi_read = &check_guntr_abi[0];
	attr_Shot_number_read = &shot_number;
	attr_Curr_shot_read = &(c_meas[0]);
	attr_B_SR_current_read = &(c0[0]);
	attr_SR_current_read = &(c1[0]);
	attr_B_SR_current_read = &(c2[0]);
	attr_I0_ST_current_read = &(c3[0]);
	attr_INJ_ST_current_read = &(c4[0]);
	attr_ACCU_ST_current_read = &(c5[0]);
	attr_LOST_ST_current_read = &(c6[0]);
	attr_Efficiency_read = &(r8[0]);
	attr_Ramping_condition_read = &ramping_conditions[0];
	attr_Part_bstB_2_1_read = &part_bstB_2_1[0];
	attr_Gun_trigger_on_read = &gun_trigger_on;
	attr_Gun_trigger_interlock_abi_read = &gun_trigger_interlock_abi;
	attr_Gun_trigger_access_abi_read = &gun_trigger_access_abi;
	attr_Version_read = &sw_version;
	attr_Ronda_fail_dr_read = &ronda_fail_dr[0];
	attr_Doors_ronda_all_read = &doors_ronda_all[0];
	attr_Memo_barr_read = &memo_barr[0];
	attr_Ronda_bo_fail_cause_read = &ronda_bo_fail_cause[0];
	attr_Ronda_int_fail_cause_read = &ronda_int_fail_cause[0];
	attr_Ronda_roof_fail_cause_read = &ronda_roof_fail_cause[0];
	attr_Ronda_dx_fail_cause_read = &ronda_dx_fail_cause[0];
	attr_Ronda_sx_fail_cause_read = &ronda_sx_fail_cause[0];
	attr_Ronda_req_cause_read = &ronda_req_cause[0];
	attr_Resettable_boards_read = &resettable_boards[0];
	attr_Disabled_boards_read = &disabled_boards[0];
	attr_Ronda_names_read = &rda_names[0];
	attr_Ctrl_names_read = &ctrlacc_names[0];
	attr_Old_eme_buttons_read = &old_eme_buttons[0];
	attr_Ronda_roof_read = &ronda_roof[0];
	attr_Ronda_int_read = &ronda_int[0];
	attr_Ronda_dx_read = &ronda_dx[0];
	attr_Ronda_sx_read = &ronda_sx[0];
	attr_Doors_all_read = &doors_all[0];
	attr_Old_doors_all_read = &old_doors_all[0];
	attr_Ronda_pt_read = &ronda_pt[0];
	attr_Puls_rda_np_read = &puls_rda_np[0];
	attr_BstBts_2_1_read = &bstbts_2_1[0];
	attr_Rf_booster_read = &rf_booster[0];
	attr_Release_key_bo_read = &release_key_bo[0];
	attr_Release_key_inj_read = &release_key_inj[0],
	attr_Timers_read = &timers[0];
	attr_Msg_est_bo_read = &msg_ext_bo;
	attr_Msg_int_bo_read = &msg_int_bo;
	attr_Msg_est_inj_read = &msg_ext_inj;
	attr_Msg_int_inj_read = &msg_int_inj;
	attr_Old_Doors_1_read = &old_doors_1[0];
	attr_Old_Doors_2_read = &old_doors_2[0];
	attr_Plc_status_read = &plc_status;
	attr_Plc_timestamp_read = &plc_timestamp;
	attr_Ronda_all_read = &ronda_all[0];
	attr_Ronda_proc_read = &ronda_proc[0];
	attr_Ronda_alm_read = &ronda_alm[0];
	attr_Ronda_rb_read = &ronda_rb[0];
	attr_Ronda_rbl_read = &ronda_rbl[0];
	attr_Ronda_barr_bst_read = &ronda_barr_bst[0];
	attr_Doors_1_read = &doors_1[0];
	attr_Doors_2_read = &doors_2[0];
	attr_Eme_buttons_read = &eme_buttons[0];
	attr_Lamp_read = &lamp[0];
	attr_Gamma_monitor_read = &gamma_monitor[0];
	attr_Keys_read = &keys[0];
	attr_Sir_buz_read = &sir_buz[0];
	attr_Part_bstB_13_1_read = &part_bstB_13_1[0];
	attr_Part_bstB_1_1_read = &part_bstB_1_1[0];
	attr_Part_bstB_1_2_read = &part_bstB_1_2[0];
	attr_Part_bstB_1_3_read = &part_bstB_1_3[0];
	attr_BstB_13_1_read = &bstb_13_1[0];
	attr_BstBts_1_1_read = &bstbts_1_1[0];
	attr_BstBts_1_2_read = &bstbts_1_2[0];
	attr_BstBts_1_3_read = &bstbts_1_3[0];
	attr_Semaph_bo_read = &semaphbo[0];
	attr_Semaph_inj_read = &semaphinj[0];
	attr_Rf_enable_read = &rfenable[0];
	attr_Bo_top_up_read = &botopup[0];
	attr_Abi_rf_read = &abirf[0];
	attr_Ring_abi_read = &ringabi[0];
	attr_Boost_stat_read = &boost_stat[0];
	attr_Machine_stat_read = &machine_stat[0];
	attr_Rf_stat_read = &rf_stat[0];
	attr_Lock_bo_read = &lock_bo[0];
	attr_Lock_inj_read = &lock_inj[0];
	attr_Alarm_door_bo_read = &alarm_door_bo[0];
	attr_Alarm_door_inj_read = &alarm_door_inj[0];
	attr_Dr_in_inj_stat_read = &dr_in_inj_stat[0];
	attr_Dr_in_inj_proc_read = &dr_in_inj_proc[0];
	attr_Dr_in_bo_proc_read = &dr_in_bo_proc[0];
	attr_Dr_in_bo_stat_read = &dr_in_bo_stat[0];
	attr_Panel_bo_read = &panelbo[0];
	attr_Panel_inj_read = &panelinj[0];
	attr_Badge_ack_bo_read = &badgeackbo;
	attr_Badge_ack_inj_read = &badgeackinj;
	attr_Posto_bo_read = &postobo[0];
	attr_Badge_bo_read = &badgebo[0];
	attr_Posto_inj_read = &postoinj[0];
	attr_Badge_inj_read = &badgeinj[0];
	//-----------------------------------

	//request for the first time rda_names and ctrl_names
	usleep(2500000);
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_50_CMD;
	DEBUG_STREAM << "PLCaccess::init_device() going to request db50" << endl;
	send_cmd();
	usleep(2000000);
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_53_CMD;
	DEBUG_STREAM << "PLCaccess::init_device() going to request db53" << endl;
	send_cmd();
	usleep(2000000);
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_121_CMD;
	DEBUG_STREAM << "PLCaccess::init_device() going to request db121" << endl;
	send_cmd();
	echoreq = 121;
	usleep(2000000);
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_122_CMD;
	DEBUG_STREAM << "PLCaccess::init_device() going to request db122" << endl;
	send_cmd();
	echoreq = 122;
	usleep(2000000);

	DEBUG_STREAM << "PLCaccess::init_device() exiting..." << endl;
}


//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void PLCaccess::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Ipaddr"));
	dev_prop.push_back(Tango::DbDatum("Plc_port"));
	dev_prop.push_back(Tango::DbDatum("Timeout"));
	dev_prop.push_back(Tango::DbDatum("Db_param"));
	dev_prop.push_back(Tango::DbDatum("Hbeat"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	PLCaccessClass	*ds_class =
		(static_cast<PLCaccessClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize Ipaddr from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  ipaddr;
	//	Try to initialize Ipaddr from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  ipaddr;
	//	And try to extract Ipaddr value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  ipaddr;

	//	Try to initialize Plc_port from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  plc_port;
	//	Try to initialize Plc_port from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  plc_port;
	//	And try to extract Plc_port value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  plc_port;

	//	Try to initialize Timeout from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  timeout;
	//	Try to initialize Timeout from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  timeout;
	//	And try to extract Timeout value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  timeout;

	//	Try to initialize Db_param from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  db_param;
	//	Try to initialize Db_param from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  db_param;
	//	And try to extract Db_param value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  db_param;

	//	Try to initialize Hbeat from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  hbeat;
	//	Try to initialize Hbeat from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  hbeat;
	//	And try to extract Hbeat value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  hbeat;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void PLCaccess::always_executed_hook()
{
	gettimeofday(&now, NULL);
	
}
//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "PLCaccess::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_BTSReal
// 
// description : 	Extract real attribute values for BTSReal acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_BTSReal(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_BTSRealTango::Attribute &attr) entering... "<< endl;
	size_t ind;
	for(ind=0; ind<NUM_FLOAT_BLK; ind++)
	{
		attr_BTSReal_read[ind] = message53.fblk[ind];
	}
	for(ind=0; ind<NUM_DWORD_BLK; ind++)
	{
		attr_BTSReal_read[ind+NUM_FLOAT_BLK] = message53.iblk[ind];
	}
	for(ind=0; ind<NUM_FLOAT2_BLK; ind++)
	{
		attr_BTSReal_read[ind+NUM_FLOAT_BLK+NUM_DWORD_BLK] = message53.fblk2[ind];
	}
	attr.set_value(attr_BTSReal_read, NUM_FLOAT_BLK+NUM_DWORD_BLK+NUM_FLOAT2_BLK);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_BTSBool
// 
// description : 	Extract real attribute values for BTSBool acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_BTSBool(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_BTSBoolTango::Attribute &attr) entering... "<< endl;
	for(size_t ind=0; ind<NUM_BOOL_BLK; ind++)
	{
		for(size_t bit=0; bit<8; bit++)
			attr_BTSBool_read[ind*8+bit] = (message53.bblk[ind]>>bit)&0x1;
	}
	attr.set_value(attr_BTSBool_read, NUM_BOOL_BLK*8);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_I_BTS_Thresh_2Gev
// 
// description : 	Extract real attribute values for I_BTS_Thresh_2Gev acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_I_BTS_Thresh_2Gev(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_I_BTS_Thresh_2GevTango::Attribute &attr) entering... "<< endl;
	for(size_t ind=0; ind<NUM_FLOAT_BLK; ind++)
	{
		attr_I_BTS_Thresh_2Gev_read[ind] = message53.fblk[ind];
	}
	attr.set_value(attr_I_BTS_Thresh_2Gev_read, NUM_FLOAT_BLK);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_I_BTS_Thresh_2_4Gev
// 
// description : 	Extract real attribute values for I_BTS_Thresh_2_4Gev acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_I_BTS_Thresh_2_4Gev(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_I_BTS_Thresh_2_4GevTango::Attribute &attr) entering... "<< endl;
	for(size_t ind=0; ind<NUM_DWORD_BLK; ind++)
	{
		attr_I_BTS_Thresh_2_4Gev_read[ind] = message53.iblk[ind];
	}
	attr.set_value(attr_I_BTS_Thresh_2_4Gev_read, NUM_DWORD_BLK);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_I_BTS
// 
// description : 	Extract real attribute values for I_BTS acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_I_BTS(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_I_BTSTango::Attribute &attr) entering... "<< endl;
	for(size_t ind=0; ind<NUM_FLOAT2_BLK-2; ind++)
	{
		attr_I_BTS_read[ind] = message53.fblk2[ind];
	}
	attr.set_value(attr_I_BTS_read, NUM_FLOAT2_BLK-2);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_I_MAX_Lost
// 
// description : 	Extract real attribute values for I_MAX_Lost acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_I_MAX_Lost(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_I_MAX_LostTango::Attribute &attr) entering... "<< endl;
	for(size_t ind=0; ind<2; ind++)
	{
		attr_I_MAX_Lost_read[ind] = message53.fblk2[NUM_FLOAT2_BLK -2 +ind];
	}
	attr.set_value(attr_I_MAX_Lost_read, 2);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Max_long_time
// 
// description : 	Extract real attribute values for Max_long_time acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Max_long_time(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Max_long_time(Tango::Attribute &attr) entering... "<< endl;
	//TODO: why is this empy???
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Max_short_time
// 
// description : 	Extract real attribute values for Max_short_time acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Max_short_time(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Max_short_time(Tango::Attribute &attr) entering... "<< endl;
	//TODO: why is this empy???
	// attr_Max_short_time_read = 0;
	// attr.set_value(attr_Max_short_time_read, CHECK_INJ_SR_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Topup_err
// 
// description : 	Extract real attribute values for Topup_err acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Topup_err(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Topup_err(Tango::Attribute &attr) entering... "<< endl;
	for(int k=0;k<TOPUP_ERR_LEN;k++){
		topup_err[k] = (((message2.blk[topup_err_byte[k]])&0x00ffff)&maskbits[topup_err_bit[k]])>>topup_err_bit[k]==0?false:true;//16bit mask
	}
//[3]->27-12->DB51.DBX67.4, [4]->27-11->DB51.DBX67.3, [5]->27-13->DB51.DBX67.5, [6]->27-8->DB51.DBX67.0, [7]->27-15->DB51.DBX67.7, [8]->27-9->DB51.DBX67.1, [9]->27-10->DB51.DBX67.2, [11]->27-14->DB51.DBX67.6, [13]->27-5->DB51.DBX66.5, [14]->27-6->DB51.DBX67.6
	topup_err[3] = (message53.bblk[2]>>7)&0x1; //bblk 2.7 -> DB53.DBX164.7 (old 27-12->DB51.DBX67.4)
	topup_err[4] = (message53.bblk[2]>>6)&0x1; //bblk 2.6 -> DB53.DBX164.6 (old 27-11->DB51.DBX67.3)
	topup_err[5] = (message53.bblk[3]>>0)&0x1; //bblk 3.0 -> DB53.DBX165.0 (old 27-13->DB51.DBX67.5)
	topup_err[6] = (message53.bblk[2]>>3)&0x1; //bblk 2.3 -> DB53.DBX164.3 (old 27-8->DB51.DBX67.0)
	topup_err[7] = (message53.bblk[3]>>2)&0x1; //bblk 3.2 -> DB53.DBX165.2 (old 27-15->DB51.DBX67.7)
	topup_err[8] = (message53.bblk[2]>>4)&0x1; //bblk 2.4 -> DB53.DBX164.4 (old 27-9->DB51.DBX67.1)
	topup_err[9] = (message53.bblk[2]>>5)&0x1; //bblk 2.5 -> DB53.DBX164.5 (old 27-10->DB51.DBX67.2)

	topup_err[11] = (message53.bblk[3]>>1)&0x1; //bblk 3.1 -> DB53.DBX165.1 (old 27-14->DB51.DBX67.6)

	topup_err[13] = (message53.bblk[2]>>0)&0x1; //bblk 2.0 -> DB53.DBX164.0 (old 27-5->DB51.DBX66.5)
	topup_err[14] = (message53.bblk[2]>>1)&0x1; //bblk 2.1 -> DB53.DBX164.1 (old 27-6->DB51.DBX66.6)

	topup_err[0] = !topup_err[0];//chiave A in booster shutdown
	topup_err[1] = !topup_err[1];//chiave C in no abi topup
//beamstopper 2.1 chiuso
	topup_err[3] = !topup_err[3];//energia magneti<>energia anello
	topup_err[4] = !topup_err[4];//corrente acc in anello <5mA
	topup_err[5] = !topup_err[5];
	topup_err[6] = !topup_err[6];
//corrente non rilevata in booster nonostante gun trigger on
	topup_err[7] = !topup_err[7];
	topup_err[8] = !topup_err[8];
	topup_err[9] = !topup_err[9];
	topup_err[10] = cmd_rst_err;
	topup_err[11] = !topup_err[11];


	attr.set_value(attr_Topup_err_read, TOPUP_ERR_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Curr_lost_short_term
// 
// description : 	Extract real attribute values for Curr_lost_short_term acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Curr_lost_short_term(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Curr_lost_short_term(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_Curr_lost_short_term_read, now, Tango::ATTR_VALID, 1,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Curr_lost_long_term
// 
// description : 	Extract real attribute values for Curr_lost_long_term acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Curr_lost_long_term(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Curr_lost_long_term(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_Curr_lost_long_term_read, now, Tango::ATTR_VALID, 1,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_LOST_LT_current
// 
// description : 	Extract real attribute values for LOST_LT_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_LOST_LT_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_LOST_LT_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_LOST_LT_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Check_topup_inj
// 
// description : 	Extract real attribute values for Check_topup_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Check_topup_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Check_topup_inj(Tango::Attribute &attr) entering... "<< endl;
	bool otp = check_topup_inj[14];//old value of topup
	for(int k=0;k<CHECK_TOPUP_INJ_LEN;k++){
		check_topup_inj[k] = (((message2.blk[check_topup_inj_byte[k]])&0x00ffff)&maskbits[check_topup_inj_bit[k]])>>check_topup_inj_bit[k]==0?false:true;//16bit mask
	}
//[8]->27-11->DB51.DBX67.3, [9]->27-13->DB51.DBX67.5, [10]->27-8->DB51.DBX67.0, [11]->27-12->DB51.DBX67.4, [12]->27-10->DB51.DBX67.2
	check_topup_inj[8] = (message53.bblk[2]>>6)&0x1; //bblk 2.6 -> DB53.DBX164.6 (old 27-11->DB51.DBX67.3)
	check_topup_inj[9] = (message53.bblk[3]>>0)&0x1; //bblk 3.0 -> DB53.DBX165.0 (old 27-13->DB51.DBX67.5)
	check_topup_inj[10] = (message53.bblk[2]>>3)&0x1; //bblk 2.3 -> DB53.DBX164.3 (old 27-8->DB51.DBX67.0)
	check_topup_inj[11] = (message53.bblk[2]>>7)&0x1; //bblk 2.7 -> DB53.DBX164.7 (old 27-12->DB51.DBX67.4)
	check_topup_inj[12] = (message53.bblk[2]>>5)&0x1; //bblk 2.5 -> DB53.DBX164.5 (old 27-10->DB51.DBX67.2)


	check_topup_inj[3] = !check_topup_inj[3];
	bool mod1rf_on = (((message2.blk[64])&0x00ffff)&maskbits[3])>>3==0?true:false;//'logic not' included
	bool sw1_m2 = (((message2.blk[64])&0x00ffff)&maskbits[7])>>7==0?false:true;
	bool sw2_m2 = (((message2.blk[64])&0x00ffff)&maskbits[8])>>8==0?false:true;
	bool mod2rf_on = (((message2.blk[64])&0x00ffff)&maskbits[5])>>5==0?true:false;//'logic not' included
	bool sw1_m1 = (((message2.blk[64])&0x00ffff)&maskbits[6])>>6==0?false:true;
	bool sw2_m1 = (((message2.blk[64])&0x00ffff)&maskbits[9])>>9==0?false:true;
	bool fbkrf_ons1 = (((message2.blk[64])&0x00ffff)&maskbits[10])>>10==0?true:false;//'logic not' included
	bool fbkrf_ons2 = (((message2.blk[64])&0x00ffff)&maskbits[11])>>11==0?true:false;//'logic not' included
	bool sect_1 = mod1rf_on&&(sw1_m2||sw2_m2);
	bool sect_2 = mod2rf_on&&(sw1_m1||sw2_m1);
	check_topup_inj[4] = sect_1||sect_2||fbkrf_ons1||fbkrf_ons2;
	check_topup_inj[5] = !check_topup_inj[5];
	if(!check_topup_inj[14]&&otp)cmd_on_topup = false;
	check_topup_inj[13] = cmd_on_topup;

	attr.set_value(attr_Check_topup_inj_read, CHECK_TOPUP_INJ_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Check_topup_test
// 
// description : 	Extract real attribute values for Check_topup_test acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Check_topup_test(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Check_topup_test(Tango::Attribute &attr) entering... "<< endl;
	for(int k=0;k<CHECK_TOPUP_TEST_LEN;k++){
		check_topup_test[k] = (((message2.blk[check_topup_test_byte[k]])&0x00ffff)&maskbits[check_topup_test_bit[k]])>>check_topup_test_bit[k]==0?false:true;//16bit mask
	}
	//[12]->27-8->DB51.DBX67.0
	check_topup_test[12] = (message53.bblk[2]>>3)&0x1; //bblk 2.3 -> DB53.DBX164.3 (old 27-8->DB51.DBX67.0)
	check_topup_test[1] = !check_topup_test[1];
	check_topup_test[2] = !check_topup_test[2];
	bool mod1rf_on = (((message2.blk[64])&0x00ffff)&maskbits[3])>>3==0?true:false;//'logic not' included
	bool sw1_m2 = (((message2.blk[64])&0x00ffff)&maskbits[7])>>7==0?false:true;
	bool sw2_m2 = (((message2.blk[64])&0x00ffff)&maskbits[8])>>8==0?false:true;
	bool mod2rf_on = (((message2.blk[64])&0x00ffff)&maskbits[5])>>5==0?true:false;//'logic not' included
	bool sw1_m1 = (((message2.blk[64])&0x00ffff)&maskbits[6])>>6==0?false:true;
	bool sw2_m1 = (((message2.blk[64])&0x00ffff)&maskbits[9])>>9==0?false:true;
	bool fbkrf_ons1 = (((message2.blk[64])&0x00ffff)&maskbits[10])>>10==0?true:false;//'logic not' included
	bool fbkrf_ons2 = (((message2.blk[64])&0x00ffff)&maskbits[11])>>11==0?true:false;//'logic not' included
	bool sect_1 = mod1rf_on&&(sw1_m2||sw2_m2);
	bool sect_2 = mod2rf_on&&(sw1_m1||sw2_m1);
	check_topup_test[3] = sect_1||sect_2||fbkrf_ons1||fbkrf_ons2;
	check_topup_test[4] = !check_topup_test[4];
	for(int i=1;i<6;i++)
		check_topup_test[6] |= check_topup_test[i];
	for(int i=7;i<10;i++)
		check_topup_test[10] |= check_topup_test[i];
	// check_topup_test[12] = !check_topup_test[12];       // <<<<=======

	attr.set_value(attr_Check_topup_test_read, CHECK_TOPUP_TEST_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Check_inj_sr
// 
// description : 	Extract real attribute values for Check_inj_sr acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Check_inj_sr(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Check_inj_sr(Tango::Attribute &attr) entering... "<< endl;
	for(int k=0;k<CHECK_INJ_SR_LEN;k++){
		check_inj_sr[k] = (((message2.blk[check_inj_sr_byte[k]])&0x00ffff)&maskbits[check_inj_sr_bit[k]])>>check_inj_sr_bit[k]==0?false:true;//16bit mask
	}
	check_inj_sr[1] = !check_inj_sr[1];	
	check_inj_sr[2] = !check_inj_sr[2];	
	bool mod1rf_on = (((message2.blk[64])&0x00ffff)&maskbits[3])>>3==0?true:false;//'logic not' included
	bool sw1_m2 = (((message2.blk[64])&0x00ffff)&maskbits[7])>>7==0?false:true;
	bool sw2_m2 = (((message2.blk[64])&0x00ffff)&maskbits[8])>>8==0?false:true;
	bool mod2rf_on = (((message2.blk[64])&0x00ffff)&maskbits[5])>>5==0?true:false;//'logic not' included
	bool sw1_m1 = (((message2.blk[64])&0x00ffff)&maskbits[6])>>6==0?false:true;
	bool sw2_m1 = (((message2.blk[64])&0x00ffff)&maskbits[9])>>9==0?false:true;
	bool fbkrf_ons1 = (((message2.blk[64])&0x00ffff)&maskbits[10])>>10==0?true:false;//'logic not' included
	bool fbkrf_ons2 = (((message2.blk[64])&0x00ffff)&maskbits[11])>>11==0?true:false;//'logic not' included
	bool sect_1 = mod1rf_on&&(sw1_m2||sw2_m2);
	bool sect_2 = mod2rf_on&&(sw1_m1||sw2_m1);
	check_inj_sr[3] = sect_1||sect_2||fbkrf_ons1||fbkrf_ons2;
	check_inj_sr[4] = !check_inj_sr[4];	
	for(int i=1;i<6;i++)
		check_inj_sr[6] |= check_inj_sr[i];
	for(int i=7;i<10;i++)
		check_inj_sr[10] |= check_inj_sr[i];
	check_inj_sr[11] = !check_inj_sr[11];	
	check_inj_sr[12] = !check_inj_sr[12];	

	attr.set_value(attr_Check_inj_sr_read, CHECK_INJ_SR_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Check_open_bst1_1
// 
// description : 	Extract real attribute values for Check_open_bst1_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Check_open_bst1_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Check_open_bst1_1(Tango::Attribute &attr) entering... "<< endl;
	for(int k=0;k<CHECK_OPN_BST1_1_LEN;k++){
		check_open_bst1_1[k] = (((message2.blk[check_open_bst1_1_byte[k]])&0x00ffff)&maskbits[check_open_bst1_1_bit[k]])>>check_open_bst1_1_bit[k]==0?false:true;//16bit mask
	}
	for(int i=4;i<6;i++)
		check_open_bst1_1[6] &= check_open_bst1_1[i];
	for(int i=7;i<10;i++)
		check_open_bst1_1[10] |= check_open_bst1_1[i];
	for(int i=1;i<4;i++)
		check_open_bst1_1[11] &= check_open_bst1_1[i];
	check_open_bst1_1[11] &= check_open_bst1_1[6];
	check_open_bst1_1[11] &= check_open_bst1_1[10];

	attr.set_value(attr_Check_open_bst1_1_read, CHECK_OPN_BST1_1_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Check_open_bst1_2
// 
// description : 	Extract real attribute values for Check_open_bst1_2 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Check_open_bst1_2(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Check_open_bst1_2(Tango::Attribute &attr) entering... "<< endl;
	for(int k=0;k<CHECK_OPN_BST1_2_LEN;k++){
		check_open_bst1_2[k] = (((message2.blk[check_open_bst1_2_byte[k]])&0x00ffff)&maskbits[check_open_bst1_2_bit[k]])>>check_open_bst1_2_bit[k]==0?false:true;//16bit mask
	}
	for(int i=4;i<6;i++)
		check_open_bst1_2[6] &= check_open_bst1_2[i];
	for(int i=7;i<10;i++)
		check_open_bst1_2[10] |= check_open_bst1_2[i];
	for(int i=1;i<4;i++)
		check_open_bst1_2[11] &= check_open_bst1_2[i];
	check_open_bst1_2[11] &= check_open_bst1_2[6];
	check_open_bst1_2[11] &= check_open_bst1_2[10];

	attr.set_value(attr_Check_open_bst1_2_read, CHECK_OPN_BST1_2_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Check_open_bst1_3
// 
// description : 	Extract real attribute values for Check_open_bst1_3 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Check_open_bst1_3(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Check_open_bst1_3(Tango::Attribute &attr) entering... "<< endl;
	for(int k=0;k<CHECK_OPN_BST1_3_LEN;k++){
		check_open_bst1_3[k] = (((message2.blk[check_open_bst1_3_byte[k]])&0x00ffff)&maskbits[check_open_bst1_3_bit[k]])>>check_open_bst1_3_bit[k]==0?false:true;//16bit mask
	}
	bool bonob = (((message2.blk[7])&0x00ffff)&maskbits[0])>>0==0?false:true;
	check_open_bst1_3[0] |= bonob;
	for(int i=0;i<2;i++)
		check_open_bst1_3[2] &= check_open_bst1_3[i];
	for(int i=4;i<6;i++)
		check_open_bst1_3[6] &= check_open_bst1_3[i];
	for(int i=7;i<10;i++)
		check_open_bst1_3[10] |= check_open_bst1_3[i];
	check_open_bst1_3[11] &= check_open_bst1_3[3];
	check_open_bst1_3[11] &= check_open_bst1_3[6];
	check_open_bst1_3[11] &= check_open_bst1_3[10];

	attr.set_value(attr_Check_open_bst1_3_read, CHECK_OPN_BST1_3_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Check_guntr_abi
// 
// description : 	Extract real attribute values for Check_guntr_abi acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Check_guntr_abi(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Check_guntr_abi(Tango::Attribute &attr) entering... "<< endl;
	for(int k=0;k<CHECK_GUN_TR_ABI_LEN;k++){
		check_guntr_abi[k] = (((message2.blk[check_guntr_abi_byte[k]])&0x00ffff)&maskbits[check_guntr_abi_bit[k]])>>check_guntr_abi_bit[k]==0?false:true;//16bit mask
	}
	//[12]->27-8->DB51.DBX67.0, [17]->27-8->DB51.DBX67.0, [20]->27-13->DB51.DBX67.5, [21]->27-9->DB51.DBX67.1, [22]->27-10->DB51.DBX67.2, [23]->27-12->DB51.DBX67.4, [24]->27-14->DB51.DBX67.6, [25]->27-8->DB51.DBX67.0, [26]->27-15->DB51.DBX67.7
	check_guntr_abi[12] = (message53.bblk[2]>>3)&0x1; //bblk 2.3 -> DB53.DBX164.2 (old 27-8->DB51.DBX67.0)
	check_guntr_abi[17] = (message53.bblk[2]>>3)&0x1; //bblk 2.3 -> DB53.DBX164.2 (old 27-8->DB51.DBX67.0)
	check_guntr_abi[20] = (message53.bblk[3]>>0)&0x1; //bblk 3.0 -> DB53.DBX165.0 (old 27-13->DB51.DBX67.5)
	check_guntr_abi[21] = (message53.bblk[2]>>4)&0x1; //bblk 2.4 -> DB53.DBX164.4 (old 27-9->DB51.DBX67.1)
	check_guntr_abi[22] = (message53.bblk[2]>>5)&0x1; //bblk 2.5 -> DB53.DBX164.5 (old 27-10->DB51.DBX67.2)
	check_guntr_abi[23] = (message53.bblk[2]>>7)&0x1; //bblk 2.7 -> DB53.DBX164.7 (old 27-12->DB51.DBX67.4)
	check_guntr_abi[24] = (message53.bblk[3]>>1)&0x1; //bblk 3.1 -> DB53.DBX165.1 (old 27-14->DB51.DBX67.6)
	check_guntr_abi[25] = (message53.bblk[2]>>3)&0x1; //bblk 2.3 -> DB53.DBX164.2 (old 27-8->DB51.DBX67.0)
	check_guntr_abi[26] = (message53.bblk[3]>>2)&0x1; //bblk 3.2 -> DB53.DBX165.2 (old 27-15->DB51.DBX67.7)


	bool bst12 = (((message2.blk[1])&0x00ffff)&maskbits[1])>>1==0?false:true;
	bool bst13 = (((message2.blk[1])&0x00ffff)&maskbits[9])>>9==0?false:true;
	check_guntr_abi[11] = bst12&&check_guntr_abi[11];
	check_guntr_abi[11] = bst13&&check_guntr_abi[11];
	for(int i=12;i<17;i++)
		check_guntr_abi[17] &= check_guntr_abi[i];
	for(int i=19;i<27;i++)
		check_guntr_abi[27] &= check_guntr_abi[i];
	/*topup_err[4] = (((message2.blk[topup_err_byte[4]])&0x00ffff)&maskbits[topup_err_bit[4]])>>topup_err_bit[4]==0?false:true;//16bit mask
	topup_err[4] = !topup_err[4];
	check_guntr_abi[27] &= !topup_err[4];*/
	check_guntr_abi[27] &= (((message2.blk[topup_err_byte[4]])&0x00ffff)&maskbits[topup_err_bit[4]])>>topup_err_bit[4]==0?false:true;//16bit mask
	for(int i=12;i<17;i++)
		check_guntr_abi[17] &= check_guntr_abi[i];
	/*check_inj_sr[13] = (((message2.blk[check_inj_sr_byte[13]])&0x00ffff)&maskbits[check_inj_sr_bit[13]])>>check_inj_sr_bit[13]==0?false:true;//16bit mask
	check_guntr_abi[17] &= check_inj_sr[13];*/
	check_guntr_abi[17] &= (((message2.blk[check_inj_sr_byte[13]])&0x00ffff)&maskbits[check_inj_sr_bit[13]])>>check_inj_sr_bit[13]==0?false:true;//16bit mask
//	check_guntr_abi[26] = !check_guntr_abi[26];
	check_guntr_abi[28] = check_guntr_abi[11];
	check_guntr_abi[28] |= check_guntr_abi[17];
	check_guntr_abi[28] |= check_guntr_abi[18];
	check_guntr_abi[28] |= check_guntr_abi[27];

	attr.set_value(attr_Check_guntr_abi_read, CHECK_GUN_TR_ABI_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_On_topup
// 
// description : 	Extract real attribute values for On_topup acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_On_topup(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_On_topup(Tango::Attribute &attr) entering... "<< endl;
	//TODO: why thi is empty???
//	attr_On_topup_read = &ontopup;
//	attr.set_value_date_quality(attr_On_topup_read, now, Tango::ATTR_VALID, 1,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Shot_number
// 
// description : 	Extract real attribute values for Shot_number acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Shot_number(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Shot_number(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_Shot_number_read, now, Tango::ATTR_VALID, 1,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Curr_shot
// 
// description : 	Extract real attribute values for Curr_shot acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Curr_shot(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Curr_shot(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_Curr_shot_read, now, Tango::ATTR_VALID, TOPUP_BLOCK_LEN,0,false);

}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_B_current
// 
// description : 	Extract real attribute values for B_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_B_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_B_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_B_SR_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_SR_current
// 
// description : 	Extract real attribute values for SR_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_SR_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_SR_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_SR_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_B_SR_current
// 
// description : 	Extract real attribute values for B_SR_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_B_SR_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_B_SR_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_B_SR_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_I0_ST_current
// 
// description : 	Extract real attribute values for I0_ST_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_I0_ST_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_I0_ST_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_I0_ST_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_INJ_ST_current
// 
// description : 	Extract real attribute values for INJ_ST_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_INJ_ST_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_INJ_ST_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_INJ_ST_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_ACCU_ST_current
// 
// description : 	Extract real attribute values for ACCU_ST_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_ACCU_ST_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_ACCU_ST_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_ACCU_ST_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_LOST_ST_current
// 
// description : 	Extract real attribute values for LOST_ST_current acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_LOST_ST_current(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_LOST_ST_current(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_LOST_ST_current_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}


//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Efficiency
// 
// description : 	Extract real attribute values for Efficiency acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Efficiency(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Efficiency(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value_date_quality(attr_Efficiency_read, now, Tango::ATTR_VALID, CURR_BUF_XLEN,0,false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ramping_condition
// 
// description : 	Extract real attribute values for Ramping_condition acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ramping_condition(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ramping_condition(Tango::Attribute &attr) entering... "<< endl;
	
	ramping_conditions[0] = ((((message2.blk[0])&0xff00)>>8)&maskbits[1])>>1==0?false:true;//bst1.1 closed
	ramping_conditions[1] = (((message2.blk[1])&0x00ff)&maskbits[1])>>1==0?false:true;//bst1.2 closed
	ramping_conditions[2] = ((((message2.blk[1])&0xff00)>>8)&maskbits[0])==0?false:true;//bst1.3 opened
	ramping_conditions[3] = ramping_conditions[0]&&ramping_conditions[1]&&ramping_conditions[2];//ramping enable

	attr.set_value(attr_Ramping_condition_read, 4, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Part_bstB_2_1
// 
// description : 	Extract real attribute values for Part_bstB_2_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Part_bstB_2_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Part_bstB_2_1(Tango::Attribute &attr) entering... "<< endl;
	part_bstB_2_1[0] = (((message2.blk[76])&0x00ff)&maskbits[0])==0?false:true;//closed sw
	part_bstB_2_1[1] = (((message2.blk[76])&0x00ff)&maskbits[1])>>1==0?false:true;//open sw
	part_bstB_2_1[2] = (((message2.blk[76])&0x00ff)&maskbits[3])>>3==0?false:true;//closed mg
	part_bstB_2_1[3] = (((message2.blk[76])&0x00ff)&maskbits[7])>>7==0?false:true;//open mg
	part_bstB_2_1[4] = true;//pss ok
	part_bstB_2_1[5] = false;//opn cmd

	attr.set_value(attr_Part_bstB_2_1_read, 6, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Gun_trigger_on
// 
// description : 	Extract real attribute values for Gun_trigger_on acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Gun_trigger_on(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Gun_trigger_on(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value(attr_Gun_trigger_on_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Gun_trigger_interlock_abi
// 
// description : 	Extract real attribute values for Gun_trigger_interlock_abi acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Gun_trigger_interlock_abi(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Gun_trigger_interlock_abi(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value(attr_Gun_trigger_interlock_abi_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Gun_trigger_access_abi
// 
// description : 	Extract real attribute values for Gun_trigger_access_abi acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Gun_trigger_access_abi(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Gun_trigger_access_abi(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value(attr_Gun_trigger_access_abi_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Version
// 
// description : 	Extract real attribute values for Version acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Version(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Version(Tango::Attribute &attr) entering... "<< endl;
	char buf[10];
   	memset (buf, 0, 10);
   	memset (sw_version, 0, BUF_LEN);

	strncpy(sw_version, "srv_ver:31\nplc_ver:", BUF_LEN);
	sprintf(buf, "%d\n",message2.spare20);
	strncat(sw_version, buf, 10);

	attr.set_value(attr_Version_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_fail_dr
// 
// description : 	Extract real attribute values for Ronda_fail_dr acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_fail_dr(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_fail_dr(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<16;j++){
		ronda_fail_dr[j] = (message2.blk[30]&maskbits[j])>>j==0?false:true;
	}
	for(int j=0;j<16;j++){
		ronda_fail_dr[j+16] = (message2.blk[31]&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Ronda_fail_dr_read, 32, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Doors_ronda_all
// 
// description : 	Extract real attribute values for Doors_ronda_all acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Doors_ronda_all(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Doors_ronda_all(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<24;j++){
		if((!doors_all[j])&&(ronda_proc[2]||ronda_dx[0]||ronda_sx[0]||\
		ronda_int[0]||ronda_roof[0]))
			doors_ronda_all[j] = false; //when a door opens, reset old_doors signal
		//otherwise does nothing, cause the set operation must be executed from end_ronda
	}

	attr.set_value(attr_Doors_ronda_all_read, 24, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Memo_barr
// 
// description : 	Extract real attribute values for Memo_barr acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Memo_barr(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Memo_barr(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value(attr_Memo_barr_read, 4, 0, false);
	
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_bo_fail_cause
// 
// description : 	Extract real attribute values for Ronda_bo_fail_cause acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_bo_fail_cause(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_bo_fail_cause(Tango::Attribute &attr) entering... "<< endl;
	ronda_bo_fail_cause[0] = ((((message2.blk[35])&0x00ff)&maskbits[7])>>7==0?false:true)&&(memo_err_inout[0]);
	ronda_bo_fail_cause[1] = ((((message2.blk[35])&0x00ff)&maskbits[7])>>7==0?false:true)&&(memo_err_inout[1]);
	ronda_bo_fail_cause[2] = (((message2.blk[35])&0x00ff)&maskbits[3])>>3==0?false:true;
	ronda_bo_fail_cause[3] = (((message2.blk[35])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_bo_fail_cause[4] = (((message2.blk[35])&0x00ff)&maskbits[6])>>6==0?false:true;
	ronda_bo_fail_cause[5] = (((message2.blk[35])&0x00ff)&maskbits[4])>>4==0?false:true;
	ronda_bo_fail_cause[6] = (((message2.blk[35])&0x00ff)&maskbits[5])>>5==0?false:true;
	ronda_bo_fail_cause[7] = (((message2.blk[34]))&maskbits[10])>>10==0?false:true;
	ronda_bo_fail_cause[8] = (((message2.blk[35]))&maskbits[1])>>1==0?false:true;
	ronda_bo_fail_cause[9] = (((message2.blk[34]))&maskbits[8])>>8==0?false:true;
	ronda_bo_fail_cause[10] = (((message2.blk[34]))&maskbits[9])>>9==0?false:true;
	ronda_bo_fail_cause[11] = (((message2.blk[34]))&maskbits[11])>>11==0?false:true;
	ronda_bo_fail_cause[12] = ((((message2.blk[35])&0x00ff)&maskbits[0])>>0==0?false:true);
	ronda_bo_fail_cause[13] = ((((message2.blk[35])&0x00ff)&maskbits[0])>>0==0?false:true);//the same above, not used
	ronda_bo_fail_cause[14] = ((((message2.blk[35])&0x00ff)&maskbits[0])>>0==0?false:true);//the same above, not used
	ronda_bo_fail_cause[15] = ((((message2.blk[35])&0x00ff)&maskbits[0])>>0==0?false:true);//the same above, not used
	ronda_bo_fail_cause[16] = (((message2.blk[34]))&maskbits[12])>>12==0?false:true;
	ronda_bo_fail_cause[17] = (((message2.blk[34]))&maskbits[13])>>13==0?false:true;
	ronda_bo_fail_cause[18] = (((message2.blk[34]))&maskbits[14])>>14==0?false:true;
	ronda_bo_fail_cause[19] = (((message2.blk[34]))&maskbits[15])>>15==0?false:true;
//	ronda_bo_fail_cause[20] = (((message2.blk[34]))&maskbits[j])>>j==0?false:true;
//TOD: check array effective lenght and usage
	attr.set_value(attr_Ronda_bo_fail_cause_read, 21, 0, false);

}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_int_fail_cause
// 
// description : 	Extract real attribute values for Ronda_int_fail_cause acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_int_fail_cause(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_int_fail_cause(Tango::Attribute &attr) entering... "<< endl;
	ronda_int_fail_cause[0] = ((((message2.blk[28])&0xff00)>>8)&maskbits[2])>>2==0?false:true;
	ronda_int_fail_cause[1] = ((((message2.blk[28])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	ronda_int_fail_cause[2] = ((((message2.blk[28])&0xff00)>>8)&maskbits[6])>>6==0?false:true;
	ronda_int_fail_cause[3] = ((((message2.blk[28])&0xff00)>>8)&maskbits[3])>>3==0?false:true;
	ronda_int_fail_cause[4] = ((((message2.blk[28])&0xff00)>>8)&maskbits[5])>>5==0?false:true;
	ronda_int_fail_cause[5] = ((((message2.blk[30])&0xff00)>>8)&maskbits[0])>>0==0?false:true;
	ronda_int_fail_cause[6] = ((((message2.blk[30])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	ronda_int_fail_cause[7] = ((((message2.blk[28])&0xff00)>>8)&maskbits[7])>>7==0?false:true;


	attr.set_value(attr_Ronda_int_fail_cause_read, 8, 0, false);

}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_roof_fail_cause
// 
// description : 	Extract real attribute values for Ronda_roof_fail_cause acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_roof_fail_cause(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_roof_fail_cause(Tango::Attribute &attr) entering... "<< endl;

	ronda_roof_fail_cause[0] = (((message2.blk[28])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_roof_fail_cause[1] = (((message2.blk[28])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_roof_fail_cause[2] = (((message2.blk[28])&0x00ff)&maskbits[6])>>6==0?false:true;
	ronda_roof_fail_cause[3] = (((message2.blk[28])&0x00ff)&maskbits[3])>>3==0?false:true;
	ronda_roof_fail_cause[4] = (((message2.blk[28])&0x00ff)&maskbits[5])>>5==0?false:true;
	ronda_roof_fail_cause[5] = (((message2.blk[28])&0x00ff)&maskbits[4])>>4==0?false:true;
	ronda_roof_fail_cause[6] = (((message2.blk[28])&0x00ff)&maskbits[7])>>7==0?false:true;

	attr.set_value(attr_Ronda_roof_fail_cause_read, 7, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_dx_fail_cause
// 
// description : 	Extract real attribute values for Ronda_dx_fail_cause acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_dx_fail_cause(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_dx_fail_cause(Tango::Attribute &attr) entering... "<< endl;

	ronda_dx_fail_cause[0] = (((message2.blk[29])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_dx_fail_cause[1] = (((message2.blk[29])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_dx_fail_cause[2] = (((message2.blk[29])&0x00ff)&maskbits[6])>>6==0?false:true;
	ronda_dx_fail_cause[3] = (((message2.blk[29])&0x00ff)&maskbits[3])>>3==0?false:true;
	ronda_dx_fail_cause[4] = (((message2.blk[29])&0x00ff)&maskbits[5])>>5==0?false:true;
	ronda_dx_fail_cause[5] = (((message2.blk[31])&0x00ff)&maskbits[0])>>0==0?false:true;
	ronda_dx_fail_cause[6] = (((message2.blk[31])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_dx_fail_cause[7] = (((message2.blk[31])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_dx_fail_cause[8] = (((message2.blk[29])&0x00ff)&maskbits[7])>>7==0?false:true;

	attr.set_value(attr_Ronda_dx_fail_cause_read, 9, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_sx_fail_cause
// 
// description : 	Extract real attribute values for Ronda_sx_fail_cause acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_sx_fail_cause(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_sx_fail_cause(Tango::Attribute &attr) entering... "<< endl;

	ronda_sx_fail_cause[0] = ((((message2.blk[29])&0xff00)>>8)&maskbits[2])>>2==0?false:true;
	ronda_sx_fail_cause[1] = ((((message2.blk[29])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	ronda_sx_fail_cause[2] = ((((message2.blk[29])&0xff00)>>8)&maskbits[6])>>6==0?false:true;
	ronda_sx_fail_cause[3] = ((((message2.blk[29])&0xff00)>>8)&maskbits[3])>>3==0?false:true;
	ronda_sx_fail_cause[4] = ((((message2.blk[29])&0xff00)>>8)&maskbits[5])>>5==0?false:true;
	ronda_sx_fail_cause[5] = ((((message2.blk[31])&0xff00)>>8)&maskbits[5])>>5==0?false:true;
	ronda_sx_fail_cause[6] = ((((message2.blk[31])&0xff00)>>8)&maskbits[4])>>4==0?false:true;
	ronda_sx_fail_cause[7] = ((((message2.blk[31])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	ronda_sx_fail_cause[8] = ((((message2.blk[31])&0xff00)>>8)&maskbits[0])>>0==0?false:true;
	ronda_sx_fail_cause[9] = ((((message2.blk[31])&0xff00)>>8)&maskbits[2])>>2==0?false:true;
	ronda_sx_fail_cause[10] = ((((message2.blk[31])&0xff00)>>8)&maskbits[3])>>3==0?false:true;
	ronda_sx_fail_cause[11] = ((((message2.blk[29])&0xff00)>>8)&maskbits[7])>>7==0?false:true;

	attr.set_value(attr_Ronda_sx_fail_cause_read, 12, 0, false);

}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_req_cause
// 
// description : 	Extract real attribute values for Ronda_req_cause acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_req_cause(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_req_cause(Tango::Attribute &attr) entering... "<< endl;
	
	ronda_req_cause[0] = -1;//reset cause
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[0])>>0==1)ronda_req_cause[0] = 1;//door bo, in
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[1])>>1==1)ronda_req_cause[0] = 0;//door inj, in
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[2])>>2==1)ronda_req_cause[0] = 3;//door bo, out
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[3])>>3==1)ronda_req_cause[0] = 2;//door inj, out
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[4])>>4==1)ronda_req_cause[0] = 5;//door bo eme
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[5])>>5==1)ronda_req_cause[0] = 4;//door inj, eme
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[6])>>6==1)ronda_req_cause[0] = 6;//puls boost
	if(((((message2.blk[15])&0xff00)>>8)&maskbits[7])>>7==1)ronda_req_cause[0] = 7;//shutd
	if(((((message2.blk[16])&0x00ff))&maskbits[0])>>0==1)ronda_req_cause[0] = 8;//rd call pc
	if(((((message2.blk[16])&0x00ff))&maskbits[1])>>1==1)ronda_req_cause[0] = 9;//startup
	if(((((message2.blk[16])&0x00ff))&maskbits[2])>>2==1)ronda_req_cause[0] = 10;//bsa11
	if(((((message2.blk[16])&0x00ff))&maskbits[3])>>3==1)ronda_req_cause[0] = 11;//bsa31
	if(((((message2.blk[16])&0x00ff))&maskbits[4])>>4==1)ronda_req_cause[0] = 12;//bsa41
	if(((((message2.blk[16])&0x00ff))&maskbits[5])>>5==1)ronda_req_cause[0] = 13;//drr
	if(((((message2.blk[16])&0x00ff))&maskbits[6])>>6==1)ronda_req_cause[0] = 16;//test in bo
	if(((((message2.blk[16])&0x00ff))&maskbits[7])>>7==1)ronda_req_cause[0] = 17;//test out bo
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[0])>>0==1)ronda_req_cause[0] = 14;//test in inj
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[1])>>1==1)ronda_req_cause[0] = 15;//test out inj
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[2])>>2==1)ronda_req_cause[0] = 18;//anomalia

	ronda_req_cause[3] = -1;//reset cause
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[3])>>3==1)ronda_req_cause[3] = 0;//cs4_1
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[4])>>4==1)ronda_req_cause[3] = 1;//a12_1
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[5])>>5==1)ronda_req_cause[3] = 2;//r3_2
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[6])>>6==1)ronda_req_cause[3] = 3;//r3_1
	if(((((message2.blk[16])&0xff00)>>8)&maskbits[7])>>7==1)ronda_req_cause[3] = 4;//startup

	ronda_req_cause[4] = -1;//reset cause
	if(((((message2.blk[17])&0x00ff))&maskbits[0])>>0==1)ronda_req_cause[4] = 7;//startup
	if(((((message2.blk[17])&0x00ff))&maskbits[1])>>1==1)ronda_req_cause[4] = 0;//csa4_2
	if(((((message2.blk[17])&0x00ff))&maskbits[2])>>2==1)ronda_req_cause[4] = 1;//a4_1
	if(((((message2.blk[17])&0x00ff))&maskbits[3])>>3==1)ronda_req_cause[4] = 2;//a5_1
	if(((((message2.blk[17])&0x00ff))&maskbits[4])>>4==1)ronda_req_cause[4] = 3;//a6_2
	if(((((message2.blk[17])&0x00ff))&maskbits[5])>>5==1)ronda_req_cause[4] = 4;//a6_1
	if(((((message2.blk[17])&0x00ff))&maskbits[6])>>6==1)ronda_req_cause[4] = 5;//a8_1
	if(((((message2.blk[17])&0x00ff))&maskbits[7])>>7==1)ronda_req_cause[4] = 6;//a10_1

	ronda_req_cause[1] = -1;//reset cause
	if(((((message2.blk[14])&0xff00)>>8)&maskbits[0])>>0==1)ronda_req_cause[1] = 0;//bsa1_1
	if(((((message2.blk[14])&0xff00)>>8)&maskbits[1])>>1==1)ronda_req_cause[1] = 1;//bsa3_1
	if(((((message2.blk[14])&0xff00)>>8)&maskbits[2])>>2==1)ronda_req_cause[1] = 2;//bsa4_1
	if(((((message2.blk[14])&0xff00)>>8)&maskbits[3])>>3==1)ronda_req_cause[1] = 3;//startup

	ronda_req_cause[2] = -1;//reset cause
	if(((((message2.blk[14])&0xff00)>>8)&maskbits[4])>>4==1)ronda_req_cause[2] = 0;//dr_r
	if(((((message2.blk[14])&0xff00)>>8)&maskbits[5])>>5==1)ronda_req_cause[2] = 1;//r3_2
	if(((((message2.blk[14])&0xff00)>>8)&maskbits[6])>>6==1)ronda_req_cause[2] = 2;//startup


	attr.set_value(attr_Ronda_req_cause_read, 5, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Resettable_boards
// 
// description : 	Extract real attribute values for Resettable_boards acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Resettable_boards(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Resettable_boards(Tango::Attribute &attr) entering... "<< endl;
	for(int y=0;y<8;y++){
		resettable_boards[y] = (((message2.blk[2])&0x00ff)&maskbits[y])>>y==0?false:true;
	}
	for(int y=8;y<16;y++){
		resettable_boards[y] = (((message2.blk[2])&0xff00)&maskbits[y])>>y==0?false:true;
	}
	for(int y=0;y<8;y++){
		resettable_boards[y+16] = (((message2.blk[3])&0x00ff)&maskbits[y])>>y==0?false:true;
	}
	for(int y=8;y<12;y++){
		resettable_boards[y+16] = (((message2.blk[3])&0xff00)&maskbits[y])>>y==0?false:true;
	}

	attr.set_value(attr_Resettable_boards_read, 28, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Disabled_boards
// 
// description : 	Extract real attribute values for Disabled_boards acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Disabled_boards(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Disabled_boards(Tango::Attribute &attr) entering... "<< endl;
	for(int y=0;y<8;y++){
		disabled_boards[y] = (((message2.blk[4])&0x00ff)&maskbits[y])>>y==0?false:true;
	}
	for(int y=8;y<16;y++){
		disabled_boards[y] = (((message2.blk[4])&0xff00)&maskbits[y])>>y==0?false:true;
	}
	for(int y=0;y<8;y++){
		disabled_boards[y+16] = (((message2.blk[5])&0x00ff)&maskbits[y])>>y==0?false:true;
	}
	for(int y=8;y<12;y++){
		disabled_boards[y+16] = (((message2.blk[5])&0xff00)&maskbits[y])>>y==0?false:true;
	}

	attr.set_value(attr_Disabled_boards_read, 28, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_names
// 
// description : 	Extract real attribute values for Ronda_names acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_names(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_names(Tango::Attribute &attr) entering... "<< endl;

	pers_name namex;
	int u=0, v=0, found=0;
	for(int k=0;k<DB_NAMES_LEN;k++){
		v=0;
		found=0;
		for(int j=0;j<NUM_BIT;j++){
			namex[v] = (unsigned char)(ronda_names.pn[k])[j];
			if((namex[v]<32)||(namex[v]>126)){
			}
			else{
				v++;
				found++;
			}
		}
		for(int h=v;h<NUM_BIT;namex[h] = ' ',h++);
		strcpy(rda_names[u], (const char*)namex);
		rda_names[u][NUM_BIT]=0;
		if(found>0)
			u++;
	}

	attr.set_value(attr_Ronda_names_read, u, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ctrl_names
// 
// description : 	Extract real attribute values for Ctrl_names acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ctrl_names(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ctrl_names(Tango::Attribute &attr) entering... "<< endl;

	pers_name namex;
	int u=0, v=0, found=0;
	for(int k=0;k<DB_NAMES_LEN;k++){
		v=0;
		found=0;
		for(int j=0;j<NUM_BIT;j++){
			namex[v] = (unsigned char)(ctrl_names.pn[k])[j];
			if((namex[v]<32)||(namex[v]>126)){
			}
			else{
				v++;
				found++;
			}
		}
		for(int h=v;h<NUM_BIT;namex[h] = ' ',h++);
		strcpy(ctrlacc_names[u], (const char*)namex);
		ctrlacc_names[u][NUM_BIT]=0;
		if(found>0)
			u++;
	}

	attr.set_value(attr_Ctrl_names_read, u, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Old_eme_buttons
// 
// description : 	Extract real attribute values for Old_eme_buttons acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Old_eme_buttons(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Old_eme_buttons(Tango::Attribute &attr) entering... "<< endl;
	if(!ronda_all[12]){//if NOT(ACK_REQ), ack already done
		for(int j=0;j<NUM_BIT;j++){
			old_eme_buttons[j] = true;
		}
	}
	old_eme_buttons[0] = (((message2.blk[60])&0x00ff)&maskbits[1])>>1==0?false:old_eme_buttons[0];
	old_eme_buttons[1] = (((message2.blk[60])&0x00ff)&maskbits[2])>>2==0?false:old_eme_buttons[1];
	old_eme_buttons[3] = (((message2.blk[79])&0x00ff)&maskbits[0])==0?false:old_eme_buttons[3];
	old_eme_buttons[4] = (((message2.blk[85])&0x00ff)&maskbits[0])==0?false:old_eme_buttons[4];
	old_eme_buttons[5] = (((message2.blk[93])&0x00ff)&maskbits[0])==0?false:old_eme_buttons[5];
	old_eme_buttons[8] = (((message2.blk[93])&0x00ff)&maskbits[1])>>1==0?false:old_eme_buttons[8];
	old_eme_buttons[6] = (((message2.blk[97])&0x00ff)&maskbits[0])==0?false:old_eme_buttons[6];
	old_eme_buttons[7] = (((message2.blk[101])&0x00ff)&maskbits[0])==0?false:old_eme_buttons[7];
	old_eme_buttons[2] = (((message2.blk[105])&0x00ff)&maskbits[0])==0?false:old_eme_buttons[2];
	old_eme_buttons[9] = (((message2.blk[113])&0x00ff)&maskbits[0])==0?false:old_eme_buttons[9];

	attr.set_value(attr_Old_eme_buttons_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_roof
// 
// description : 	Extract real attribute values for Ronda_roof acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_roof(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_roof(Tango::Attribute &attr) entering... "<< endl;
	for(int i=0;i<8;i++){
		ronda_roof[i] = (((message2.blk[28])&0x00ff)&maskbits[i])>>i==0?false:true;
	}

	attr.set_value(attr_Ronda_roof_read, 8, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_int
// 
// description : 	Extract real attribute values for Ronda_int acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_int(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_int(Tango::Attribute &attr) entering... "<< endl;

	for(int i=0;i<8;i++){
		ronda_int[i] = ((((message2.blk[28])&0xff00)>>8)&maskbits[i])>>i==0?false:true;
	}

	attr.set_value(attr_Ronda_int_read, 8, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_dx
// 
// description : 	Extract real attribute values for Ronda_dx acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_dx(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_dx(Tango::Attribute &attr) entering... "<< endl;
	for(int i=0;i<8;i++){
		ronda_dx[i] = (((message2.blk[29])&0x00ff)&maskbits[i])>>i==0?false:true;
	}

	attr.set_value(attr_Ronda_dx_read, 8, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_sx
// 
// description : 	Extract real attribute values for Ronda_sx acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_sx(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_sx(Tango::Attribute &attr) entering... "<< endl;
	for(int i=0;i<8;i++){
		ronda_sx[i] = ((((message2.blk[29])&0xff00)>>8)&maskbits[i])>>i==0?false:true;
	}

	attr.set_value(attr_Ronda_sx_read, 8, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Doors_all
// 
// description : 	Extract real attribute values for Doors_all acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Doors_all(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Doors_all(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<3;j++){
		doors_all[j] = ((((message2.blk[55])&0xff00)>>8)&maskbits[j+5])>>(j+5)==0?false:true;
	}
	for(int j=0;j<5;j++){
		doors_all[3+j] = (((message2.blk[56])&0x00ff)&maskbits[j])>>j==0?false:true;
	}
	doors_all[8] = (((message2.blk[60])&0x00ff)&maskbits[0])==0?false:true;
	doors_all[10] = (((message2.blk[60])&0x00ff)&maskbits[7])>>7==0?false:true;
	doors_all[9] = ((((message2.blk[60])&0xff00)>>8)&maskbits[0])==0?false:true;
	doors_all[11] = (((message2.blk[70])&0x00ff)&maskbits[0])==0?false:true;
	doors_all[12] = (((message2.blk[70])&0x00ff)&maskbits[1])>>1==0?false:true;
	for(int j=0;j<3;j++){
		doors_all[j+13] = (((message2.blk[74])&0x00ff)&maskbits[j])>>j==0?false:true;
	}
	doors_all[16] = (((message2.blk[93])&0x00ff)&maskbits[2])>>2==0?false:true;
	doors_all[17] = (((message2.blk[101])&0x00ff)&maskbits[1])>>1==0?false:true;
	doors_all[18] = (((message2.blk[105])&0x00ff)&maskbits[1])>>1==0?false:true;
	doors_all[19] = (((message2.blk[105])&0x00ff)&maskbits[2])>>2==0?false:true;
	for(int j=0;j<4;j++){
		doors_all[j+20] = ((((message2.blk[34])&0xff00)>>8)&maskbits[j])>>j==0?false:true;
	}


	attr.set_value(attr_Doors_all_read, 24, 0, false);
	
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Old_doors_all
// 
// description : 	Extract real attribute values for Old_doors_all acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Old_doors_all(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Old_doors_all(Tango::Attribute &attr) entering... "<< endl;
	if(ronda_all[0]){//ronda booster
		old_doors_all[3] = true; //when ronda is ok, closed doors become white
		old_doors_all[8] = true; 
	}
	if(ronda_all[1]){//ronda dx
		old_doors_all[10] = true; 
		old_doors_all[13] = true; 
		old_doors_all[14] = true; 
		old_doors_all[15] = true; 
	}
	if(ronda_all[2]){//ronda sx
		old_doors_all[9] = true; 
		old_doors_all[4] = true; 
		old_doors_all[5] = true; 
		old_doors_all[7] = true; 
		old_doors_all[6] = true; 
		old_doors_all[12] = true; 
		old_doors_all[11] = true; 
	}
	if(ronda_all[3]){//ronda chiostrina
		old_doors_all[16] = true; 
		old_doors_all[17] = true; 
		old_doors_all[18] = true; 
	}
	if(ronda_all[4]){//ronda tetto
		old_doors_all[19] = true; 
		old_doors_all[15] = true; 
	}
	for(int j=0;j<24;j++){
		if(!doors_all[j])
			old_doors_all[j] = false; //when a door opens, reset old_doors signal
		//otherwise does nothing, cause the set operation must be executed from end_ronda
	}

	attr.set_value(attr_Old_doors_all_read, 24, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_pt
// 
// description : 	Extract real attribute values for Ronda_pt acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_pt(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_pt(Tango::Attribute &attr) entering... "<< endl;
	ronda_pt[0] = ((((message2.blk[56])&0xff00)>>8)&maskbits[2])>>2==0?false:true;
	ronda_pt[1] = ((((message2.blk[56])&0xff00)>>8)&maskbits[3])>>3==0?false:true;
	ronda_pt[2] = ((((message2.blk[60])&0xff00)>>8)&maskbits[2])>>2==0?false:true;
	ronda_pt[3] = ((((message2.blk[60])&0xff00)>>8)&maskbits[3])>>3==0?false:true;

	attr.set_value(attr_Ronda_pt_read, 4, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Puls_rda_np
// 
// description : 	Extract real attribute values for Puls_rda_np acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Puls_rda_np(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Puls_rda_np(Tango::Attribute &attr) entering... "<< endl;
	puls_rda_np[0] = (((message2.blk[118])&0x00ff)&maskbits[1])>>1==0?false:true;
	puls_rda_np[1] = (((message2.blk[120])&0x00ff)&maskbits[0])==0?false:true;
	puls_rda_np[2] = (((message2.blk[120])&0x00ff)&maskbits[1])>>1==0?false:true;
	puls_rda_np[3] = (((message2.blk[151])&0x00ff)&maskbits[0])==0?false:true;
	puls_rda_np[4] = (((message2.blk[28])&0x00ff)&maskbits[6])>>6==0?false:true;
	puls_rda_np[5] = (((message2.blk[155])&0x00ff)&maskbits[0])==0?false:true;
	for(int j=6;j<NUM_POSTO;j++){
		puls_rda_np[j] = true;
	}

	attr.set_value(attr_Puls_rda_np_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_BstBts_2_1
// 
// description : 	Extract real attribute values for BstBts_2_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_BstBts_2_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_BstBts_2_1(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<8;j++){
		bstbts_2_1[j] = false;
	}
	bstbts_2_1[0] = (((message2.blk[132])&0x00ff)&maskbits[1])>>1==0?false:true;//cum open
	bstbts_2_1[1] = (((message2.blk[132])&0x00ff)&maskbits[0])==0?false:true;//cum cld
	bstbts_2_1[2] = (((message2.blk[76])&0x00ff)&maskbits[2])>>2==0?true:false;//NOT(BST_OK) generic error
	bstbts_2_1[3] = (((message2.blk[76])&0x00ff)&maskbits[2])>>2==0?true:false;//NOT(BST_OK) generic error
	bstbts_2_1[4] = false;//app open cmd
	bstbts_2_1[5] = false;//error switch
	bstbts_2_1[7] = (((message2.blk[76])&0x00ff)&maskbits[2])>>2==0?false:true;//enabled (funz ok)

	attr.set_value(attr_BstBts_2_1_read, 8, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Rf_booster
// 
// description : 	Extract real attribute values for Rf_booster acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Rf_booster(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Rf_booster(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		rf_booster[j] = (message2.blk[107]&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Rf_booster_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Release_key_bo
// 
// description : 	Extract real attribute values for Release_key_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Release_key_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Release_key_bo(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		release_key_bo[j] = (message2.blk[117]&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Release_key_bo_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Release_key_inj
// 
// description : 	Extract real attribute values for Release_key_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Release_key_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Release_key_inj(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		release_key_inj[j] = (message2.blk[119]&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Release_key_inj_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Timers
// 
// description : 	Extract real attribute values for Timers acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Timers(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Timers(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<4;j++){
		timers[j] = ((((message2.blk[8])&0xff00)>>8)&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Timers_read, 4, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Msg_est_bo
// 
// description : 	Extract real attribute values for Msg_est_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Msg_est_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Msg_est_bo(Tango::Attribute &attr) entering... "<< endl;
	msg_ext_bo = message2.msg_est_bo;

	attr.set_value(attr_Msg_est_bo_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Msg_int_bo
// 
// description : 	Extract real attribute values for Msg_int_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Msg_int_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Msg_int_bo(Tango::Attribute &attr) entering... "<< endl;
	msg_int_bo = message2.msg_int_bo;
	attr.set_value(attr_Msg_int_bo_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Msg_est_inj
// 
// description : 	Extract real attribute values for Msg_est_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Msg_est_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Msg_est_inj(Tango::Attribute &attr) entering... "<< endl;
	msg_ext_inj = message2.msg_est_inj;

	attr.set_value(attr_Msg_est_inj_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Msg_int_inj
// 
// description : 	Extract real attribute values for Msg_int_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Msg_int_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Msg_int_inj(Tango::Attribute &attr) entering... "<< endl;
	msg_int_inj = message2.msg_int_inj;

	attr.set_value(attr_Msg_int_inj_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Old_Doors_1
// 
// description : 	Extract real attribute values for Old_Doors_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Old_Doors_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Old_Doors_1(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<13;j++){
		old_doors_1[j] = ronda_all[0]&&doors_1[j];
	}
	for(int j=13;j<NUM_BIT;j++){
		old_doors_1[j] = true;
	}

	attr.set_value(attr_Old_Doors_1_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Old_Doors_2
// 
// description : 	Extract real attribute values for Old_Doors_2 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Old_Doors_2(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Old_Doors_2(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<7;j++){
		old_doors_2[j] = ronda_all[0]&&doors_2[j];
	}
	for(int j=0;j<4;j++){//signals already inside of ronda_all[0]
		old_doors_2[j+7] = true;
	}
	for(int j=10;j<NUM_BIT;j++){
		old_doors_2[j] = true;
	}

	attr.set_value(attr_Old_Doors_2_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Plc_status
// 
// description : 	Extract real attribute values for Plc_status acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Plc_status(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Plc_status(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value(attr_Plc_status_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Plc_timestamp
// 
// description : 	Extract real attribute values for Plc_timestamp acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Plc_timestamp(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Plc_timestamp(Tango::Attribute &attr) entering... "<< endl;

	attr.set_value(attr_Plc_timestamp_read, 1, 0, false);
	
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_all
// 
// description : 	Extract real attribute values for Ronda_all acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_all(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_all(Tango::Attribute &attr) entering... "<< endl;
	for(int y=0;y<8;y++){
		ronda_all[y] = (((message2.blk[32])&0x00ff)&maskbits[y])>>y==0?false:true;
	}
	ronda_all[8] = (((message2.blk[6])&0x00ff)&maskbits[0])==0?false:true;
	ronda_all[9] = ((((message2.blk[32])&0xff00)>>8)&maskbits[3])>>3==0?false:true;
	ronda_all[10] = ((((message2.blk[32])&0xff00)>>8)&maskbits[2])>>2==0?false:true;
	for(int y=3;y<8;y++){
		ronda_all[8+y] = (((message2.blk[6])&0x00ff)&maskbits[y])>>y==0?false:true;
	}

	attr.set_value(attr_Ronda_all_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_proc
// 
// description : 	Extract real attribute values for Ronda_proc acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_proc(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_proc(Tango::Attribute &attr) entering... "<< endl;
	ronda_proc[0] = (((message2.blk[34]))&maskbits[0])==0?false:true;
	ronda_proc[1] = (((message2.blk[35]))&maskbits[1])>>1==0?false:true;
	for(int j=2;j<NUM_BIT;j++){
		ronda_proc[j] = (((message2.blk[34]))&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Ronda_proc_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_alm
// 
// description : 	Extract real attribute values for Ronda_alm acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_alm(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_alm(Tango::Attribute &attr) entering... "<< endl;
	ronda_alm[0] = (((message2.blk[35])&0x00ff)&maskbits[0])==0?false:true;
	ronda_alm[1] = (((message2.blk[34])&0x00ff)&maskbits[1])>>1==0?false:true;
	for(int j=2;j<NUM_POSTO;j++){
		ronda_alm[j] = (((message2.blk[35])&0x00ff)&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Ronda_alm_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_rb
// 
// description : 	Extract real attribute values for Ronda_rb acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_rb(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_rb(Tango::Attribute &attr) entering... "<< endl;
	ronda_rb[0] = (((message2.blk[72])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rb[1] = ((((message2.blk[78])&0xff00)>>8)&maskbits[0])==0?false:true;
	ronda_rb[2] = (((message2.blk[81])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rb[3] = (((message2.blk[87])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rb[4] = (((message2.blk[95])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_rb[5] = (((message2.blk[95])&0x00ff)&maskbits[3])>>3==0?false:true;
	ronda_rb[6] = (((message2.blk[81])&0x00ff)&maskbits[3])>>3==0?false:true;
	ronda_rb[7] = ((((message2.blk[95])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	ronda_rb[8] = (((message2.blk[99])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rb[9] = (((message2.blk[103])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rb[10] = (((message2.blk[109])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rb[11] = (((message2.blk[81])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_rb[12] = (((message2.blk[155])&0x00ff)&maskbits[0])==0?false:true;
	for(int j=13;j<NUM_BIT;j++){
		ronda_rb[j] = true;
	}

	attr.set_value(attr_Ronda_rb_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_rbl
// 
// description : 	Extract real attribute values for Ronda_rbl acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_rbl(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_rbl(Tango::Attribute &attr) entering... "<< endl;
	ronda_rbl[0] = (((message2.blk[127])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[1] = ((((message2.blk[131])&0xff00)>>8)&maskbits[0])==0?false:true;
	ronda_rbl[2] = (((message2.blk[133])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[3] = (((message2.blk[137])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[4] = (((message2.blk[141])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[5] = (((message2.blk[141])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_rbl[6] = ((((message2.blk[133])&0x00ff))&maskbits[3])>>3==0?false:true;
	ronda_rbl[7] = ((((message2.blk[141])&0x00ff))&maskbits[3])>>3==0?false:true;
	ronda_rbl[8] = (((message2.blk[143])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[9] = ((((message2.blk[145])&0xff00)>>8)&maskbits[0])==0?false:true;//303.0
	ronda_rbl[11] = (((message2.blk[151])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[10] = (((message2.blk[133])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_rbl[12] = (((message2.blk[155])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[13] = (((message2.blk[120])&0x00ff)&maskbits[0])==0?false:true;
	ronda_rbl[14] = (((message2.blk[120])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_rbl[15] = (((message2.blk[118])&0x00ff)&maskbits[1])>>1==0?false:true;

	attr.set_value(attr_Ronda_rbl_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ronda_barr_bst
// 
// description : 	Extract real attribute values for Ronda_barr_bst acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ronda_barr_bst(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ronda_barr_bst(Tango::Attribute &attr) entering... "<< endl;
	ronda_barr_bst[0] = (((message2.blk[81])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_barr_bst[1] = ((((message2.blk[87])&0x00ff))&maskbits[1])>>1==0?false:true;
	ronda_barr_bst[2] = (((message2.blk[99])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_barr_bst[3] = (((message2.blk[103])&0x00ff)&maskbits[1])>>1==0?false:true;
	ronda_barr_bst[4] = (((message2.blk[87])&0x00ff)&maskbits[2])>>2==0?false:true;
	ronda_barr_bst[5] = (((message2.blk[95])&0x00ff)&maskbits[0])==0?false:true;
	ronda_barr_bst[6] = ((((message2.blk[95])&0x00ff))&maskbits[1])>>1==0?false:true;
	ronda_barr_bst[7] = ((((message2.blk[115])&0x00ff))&maskbits[1])>>1==0?false:true;
	for(int j=8;j<NUM_BIT;j++){
		ronda_barr_bst[j] = true;
	}

	attr.set_value(attr_Ronda_barr_bst_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Doors_1
// 
// description : 	Extract real attribute values for Doors_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Doors_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Doors_1(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<3;j++){
		doors_1[j] = ((((message2.blk[55])&0xff00)>>8)&maskbits[j+5])>>(j+5)==0?false:true;
	}
	for(int j=0;j<5;j++){
		doors_1[3+j] = (((message2.blk[56])&0x000f)&maskbits[j])>>j==0?false:true;
	}
	doors_1[8] = (((message2.blk[60])&0x00ff)&maskbits[0])==0?false:true;
	doors_1[9] = (((message2.blk[60])&0x00ff)&maskbits[7])>>7==0?false:true;
	doors_1[10] = ((((message2.blk[60])&0xff00)>>8)&maskbits[0])==0?false:true;
	doors_1[11] = (((message2.blk[70])&0x00ff)&maskbits[0])==0?false:true;
	doors_1[12] = (((message2.blk[70])&0x00ff)&maskbits[1])>>1==0?false:true;
	for(int j=13;j<NUM_BIT;j++){
		doors_1[j] = true;
	}

	attr.set_value(attr_Doors_1_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Doors_2
// 
// description : 	Extract real attribute values for Doors_2 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Doors_2(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Doors_2(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<3;j++){
		doors_2[j] = (((message2.blk[148])&0x00ff)&maskbits[j])>>j==0?false:true;
	}
	doors_2[3] = (((message2.blk[93])&0x00ff)&maskbits[2])>>2==0?false:true;
	doors_2[4] = (((message2.blk[101])&0x00ff)&maskbits[1])>>1==0?false:true;
	doors_2[5] = (((message2.blk[105])&0x00ff)&maskbits[1])>>1==0?false:true;
	doors_2[6] = (((message2.blk[105])&0x00ff)&maskbits[2])>>2==0?false:true;
	for(int j=0;j<4;j++){
		doors_2[j+7] = ((((message2.blk[34])&0xff00)>>8)&maskbits[j])>>j==0?false:true;
	}
	for(int j=10;j<NUM_BIT;j++){
		doors_2[j] = true;
	}

	attr.set_value(attr_Doors_2_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Eme_buttons
// 
// description : 	Extract real attribute values for Eme_buttons acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Eme_buttons(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Eme_buttons(Tango::Attribute &attr) entering... "<< endl;
	eme_buttons[0] = (((message2.blk[60])&0x00ff)&maskbits[1])>>1==0?false:true;
	eme_buttons[1] = (((message2.blk[60])&0x00ff)&maskbits[2])>>2==0?false:true;
	eme_buttons[3] = (((message2.blk[79])&0x00ff)&maskbits[0])==0?false:true;
	eme_buttons[4] = (((message2.blk[85])&0x00ff)&maskbits[0])==0?false:true;
	eme_buttons[5] = (((message2.blk[93])&0x00ff)&maskbits[0])==0?false:true;
	eme_buttons[8] = (((message2.blk[93])&0x00ff)&maskbits[1])>>1==0?false:true;
	eme_buttons[6] = (((message2.blk[97])&0x00ff)&maskbits[0])==0?false:true;
	eme_buttons[7] = (((message2.blk[101])&0x00ff)&maskbits[0])==0?false:true;
	eme_buttons[2] = (((message2.blk[105])&0x00ff)&maskbits[0])==0?false:true;
	eme_buttons[9] = (((message2.blk[113])&0x00ff)&maskbits[0])==0?false:true;
	for(int j=10;j<NUM_BIT;j++){
		eme_buttons[j] = true;
	}

	attr.set_value(attr_Eme_buttons_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Lamp
// 
// description : 	Extract real attribute values for Lamp acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Lamp(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Lamp(Tango::Attribute &attr) entering... "<< endl;
	lamp[0] = (((message2.blk[53])&0x00ff)&maskbits[1])>>1==0?false:true;
	lamp[1] = (((message2.blk[55])&0x00ff)&maskbits[2])>>2==0?false:true;
	lamp[2] = ((((message2.blk[117])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	lamp[3] = ((((message2.blk[117])&0xff00)>>8)&maskbits[7])>>7==0?false:true;
	lamp[4] = (((message2.blk[118])&0x00ff)&maskbits[0])>>0==0?false:true;
	lamp[5] = ((((message2.blk[119])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	lamp[6] = ((((message2.blk[119])&0xff00)>>8)&maskbits[7])>>7==0?false:true;
	lamp[7] = (((message2.blk[120])&0x00ff)&maskbits[2])>>2==0?false:true;
	lamp[8] = (((message2.blk[127])&0x00ff)&maskbits[3])>>3==0?false:true;
	lamp[9] = ((((message2.blk[131])&0xff00)>>8)&maskbits[3])>>3==0?false:true;
	for(int j=10;j<NUM_BIT;j++){
		lamp[j] = true;
	}

	attr.set_value(attr_Lamp_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Gamma_monitor
// 
// description : 	Extract real attribute values for Gamma_monitor acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Gamma_monitor(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Gamma_monitor(Tango::Attribute &attr) entering... "<< endl;
	gamma_monitor[0] = ((((message2.blk[32])&0xff00)>>8)&maskbits[5])>>5==0?false:true;
	gamma_monitor[1] = ((((message2.blk[32])&0xff00)>>8)&maskbits[6])>>6==0?false:true;
	gamma_monitor[2] = ((((message2.blk[32])&0xff00)>>8)&maskbits[4])>>4==0?false:true;

	attr.set_value(attr_Gamma_monitor_read, 3, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Keys
// 
// description : 	Extract real attribute values for Keys acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Keys(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Keys(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<3;j++){
		keys[j] = (((message2.blk[68])&0x00ff)&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Keys_read, 3, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Sir_buz
// 
// description : 	Extract real attribute values for Sir_buz acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Sir_buz(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Sir_buz(Tango::Attribute &attr) entering... "<< endl;
	sir_buz[0] = (((message2.blk[118])&0x00ff)&maskbits[4])>>4==0?false:true;
	sir_buz[1] = ((((message2.blk[119])&0xff00)>>8)&maskbits[5])>>5==0?false:true;
	sir_buz[2] = (((message2.blk[120])&0x00ff)&maskbits[4])>>4==0?false:true;
	sir_buz[3] = (((message2.blk[133])&0x00ff)&maskbits[1])>>1==0?false:true;
	sir_buz[4] = (((message2.blk[137])&0x00ff)&maskbits[4])>>4==0?false:true;
	sir_buz[5] = (((message2.blk[143])&0x00ff)&maskbits[1])>>1==0?false:true;
	for(int j=6;j<NUM_POSTO;j++){
		sir_buz[j] = true;
	}

	attr.set_value(attr_Sir_buz_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Part_bstB_13_1
// 
// description : 	Extract real attribute values for Part_bstB_13_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Part_bstB_13_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Part_bstB_13_1(Tango::Attribute &attr) entering... "<< endl;
	part_bstB_13_1[0] = (((message2.blk[83])&0x00ff)&maskbits[0])==0?false:true;
	part_bstB_13_1[1] = (((message2.blk[83])&0x00ff)&maskbits[1])>>1==0?false:true;
	part_bstB_13_1[2] = (((message2.blk[91])&0x00ff)&maskbits[0])==0?false:true;//bst chiuso contatto magnetico
	part_bstB_13_1[3] = (((message2.blk[91])&0x00ff)&maskbits[4])>>4==0?false:true;
	part_bstB_13_1[4] = ((((message2.blk[95])&0xff00)>>8)&maskbits[0])==0?false:true;//press
	part_bstB_13_1[5] = ((((message2.blk[139])&0xff00)>>8)&maskbits[2])>>2==0?false:true;//comando apertura bst

	attr.set_value(attr_Part_bstB_13_1_read, 6, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Part_bstB_1_1
// 
// description : 	Extract real attribute values for Part_bstB_1_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Part_bstB_1_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Part_bstB_1_1(Tango::Attribute &attr) entering... "<< endl;
	part_bstB_1_1[0] = (((message2.blk[83])&0x00ff)&maskbits[2])>>2==0?false:true;//bst chiuso contatto switch
	part_bstB_1_1[1] = (((message2.blk[83])&0x00ff)&maskbits[3])>>3==0?false:true;//bst aperto contatto switch
	part_bstB_1_1[2] = (((message2.blk[91])&0x00ff)&maskbits[1])>>1==0?false:true;//bst chiuso contatto magnetico
	part_bstB_1_1[3] = (((message2.blk[91])&0x00ff)&maskbits[5])>>5==0?false:true;//bst aperto contatto magnetico
	part_bstB_1_1[4] = (((message2.blk[95])&0x00ff)&maskbits[0])==0?false:true;//press
	part_bstB_1_1[5] = ((((message2.blk[139])&0xff00)>>8)&maskbits[0])==0?false:true;//comando apertura bst

	attr.set_value(attr_Part_bstB_1_1_read, 6, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Part_bstB_1_2
// 
// description : 	Extract real attribute values for Part_bstB_1_2 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Part_bstB_1_2(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Part_bstB_1_2(Tango::Attribute &attr) entering... "<< endl;
	part_bstB_1_2[0] = (((message2.blk[89])&0x00ff)&maskbits[0])==0?false:true;
	part_bstB_1_2[1] = (((message2.blk[89])&0x00ff)&maskbits[1])>>1==0?false:true;
	part_bstB_1_2[2] = (((message2.blk[91])&0x00ff)&maskbits[2])>>2==0?false:true;//bst chiuso contatto magnetico
	part_bstB_1_2[3] = (((message2.blk[91])&0x00ff)&maskbits[6])>>6==0?false:true;
	part_bstB_1_2[4] = (((message2.blk[95])&0x00ff)&maskbits[1])>>1==0?false:true;//press
	part_bstB_1_2[5] = ((((message2.blk[139])&0xff00)>>8)&maskbits[1])>>1==0?false:true;//comando apertura bst

	attr.set_value(attr_Part_bstB_1_2_read, 6, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Part_bstB_1_3
// 
// description : 	Extract real attribute values for Part_bstB_1_3 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Part_bstB_1_3(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Part_bstB_1_3(Tango::Attribute &attr) entering... "<< endl;
	part_bstB_1_3[0] = (((message2.blk[111])&0x00ff)&maskbits[0])==0?false:true;
	part_bstB_1_3[1] = (((message2.blk[111])&0x00ff)&maskbits[1])>>1==0?false:true;
	part_bstB_1_3[2] = (((message2.blk[113])&0x00ff)&maskbits[1])>>1==0?false:true;
	part_bstB_1_3[3] = (((message2.blk[113])&0x00ff)&maskbits[5])>>5==0?false:true;
	part_bstB_1_3[4] = (((message2.blk[115])&0x00ff)&maskbits[1])>>1==0?false:true;//press
	part_bstB_1_3[5] = ((((message2.blk[153])&0xff00)>>8)&maskbits[0])==0?false:true;//comando apertura bst
	attr.set_value(attr_Part_bstB_1_3_read, 6, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_BstB_13_1
// 
// description : 	Extract real attribute values for BstB_13_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_BstB_13_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_BstB_13_1(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		bstb_13_1[j] = (((message2.blk[0])&0x00ff)&maskbits[j])>>j==0?false:true;
	}
	bstb_13_1[2] = ((((message2.blk[33])&0x00ff))&maskbits[0])==0?false:true;
	bstb_13_1[3] = ((((message2.blk[33])&0x00ff))&maskbits[1])>>1==0?false:true;
	attr.set_value(attr_BstB_13_1_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_BstBts_1_1
// 
// description : 	Extract real attribute values for BstBts_1_1 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_BstBts_1_1(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_BstBts_1_1(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		bstbts_1_1[j] = ((((message2.blk[0])&0xff00)>>8)&maskbits[j])>>j==0?false:true;
	}
	bstbts_1_1[2] = ((((message2.blk[33])&0x00ff))&maskbits[2])>>2==0?false:true;
	bstbts_1_1[3] = ((((message2.blk[33])&0x00ff))&maskbits[3])>>3==0?false:true;
	attr.set_value(attr_BstBts_1_1_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_BstBts_1_2
// 
// description : 	Extract real attribute values for BstBts_1_2 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_BstBts_1_2(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_BstBts_1_2(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		bstbts_1_2[j] = (((message2.blk[1])&0x00ff)&maskbits[j])>>j==0?false:true;
	}
	bstbts_1_2[2] = ((((message2.blk[33])&0x00ff))&maskbits[4])>>4==0?false:true;
	bstbts_1_2[3] = ((((message2.blk[33])&0x00ff))&maskbits[5])>>5==0?false:true;
	attr.set_value(attr_BstBts_1_2_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_BstBts_1_3
// 
// description : 	Extract real attribute values for BstBts_1_3 acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_BstBts_1_3(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_BstBts_1_3(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		bstbts_1_3[j] = ((((message2.blk[1])&0xff00)>>8)&maskbits[j])>>j==0?false:true;
	}
	bstbts_1_3[2] = ((((message2.blk[33])&0x00ff))&maskbits[6])>>6==0?false:true;
	bstbts_1_3[3] = ((((message2.blk[33])&0x00ff))&maskbits[7])>>7==0?false:true;
	attr.set_value(attr_BstBts_1_3_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Semaph_bo
// 
// description : 	Extract real attribute values for Semaph_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Semaph_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Semaph_bo(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		semaphbo[j] = (((message2.blk[117])>>8)&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Semaph_bo_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Semaph_inj
// 
// description : 	Extract real attribute values for Semaph_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Semaph_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Semaph_inj(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_POSTO;j++){
		semaphinj[j] = (((message2.blk[119])>>8)&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Semaph_inj_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Rf_enable
// 
// description : 	Extract real attribute values for Rf_enable acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Rf_enable(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Rf_enable(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<7;j++){
		rfenable[j] = ((message2.blk[123])&maskbits[j])>>j==0?false:true;
	}
	rfenable[7] = (((message2.blk[109])&0x00ff)&maskbits[2])>>2==0?false:true;
	rfenable[8] = ((((message2.blk[6])&0xff00)>>8)&maskbits[7])>>7==0?false:true;
	attr.set_value(attr_Rf_enable_read, 9, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Bo_top_up
// 
// description : 	Extract real attribute values for Bo_top_up acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Bo_top_up(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Bo_top_up(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<4;j++){
		botopup[j] = ((message2.blk[129])&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Bo_top_up_read, 4, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Abi_rf
// 
// description : 	Extract real attribute values for Abi_rf acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Abi_rf(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Abi_rf(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<2;j++){
		abirf[j] = ((message2.blk[147])&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Abi_rf_read, 2, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Ring_abi
// 
// description : 	Extract real attribute values for Ring_abi acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Ring_abi(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Ring_abi(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<2;j++){
		ringabi[j] = ((message2.blk[70])&maskbits[j+2])>>(j+2)==0?false:true;
		ringabi[j+2] = ((((message2.blk[72])&0xff00)>>8)&maskbits[j])>>j==0?false:true;
	}
	ringabi[4] = ((message2.blk[74])&maskbits[3])>>3==0?false:true;
	ringabi[5] = ((message2.blk[76])&maskbits[6])>>6==0?false:true;
	ringabi[6] = ((message2.blk[74])&maskbits[7])>>7==0?false:true;
	attr.set_value(attr_Ring_abi_read, 7, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Boost_stat
// 
// description : 	Extract real attribute values for Boost_stat acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Boost_stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Boost_stat(Tango::Attribute &attr) entering... "<< endl;
	boost_stat[0] = ((((message2.blk[32])&0xff00)>>8)&maskbits[0])==0?false:true;
	boost_stat[1] = ((((message2.blk[32])&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	for(int y=2;y<8;y++){
		boost_stat[y] = ((((message2.blk[6])&0xff00)>>8)&maskbits[y])>>y==0?false:true;
	}
	for(int y=0;y<8;y++){
		boost_stat[8+y] = (((message2.blk[7])&0x00ff)&maskbits[y])>>y==0?false:true;
	}
	attr.set_value(attr_Boost_stat_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Machine_stat
// 
// description : 	Extract real attribute values for Machine_stat acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Machine_stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Machine_stat(Tango::Attribute &attr) entering... "<< endl;
	for(int y=0;y<8;y++){
		machine_stat[y] = ((((message2.blk[7])&0xff00)>>8)&maskbits[y])>>y==0?false:true;
	}
	for(int y=0;y<8;y++){
		machine_stat[8+y] = (((message2.blk[8])&0x00ff)&maskbits[y])>>y==0?false:true;
	}
	attr.set_value(attr_Machine_stat_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Rf_stat
// 
// description : 	Extract real attribute values for Rf_stat acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Rf_stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Rf_stat(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<7;j++){
		rf_stat[j] = (message2.blk[64]&maskbits[j])>>j==0?false:true;
	}
		rf_stat[7] = (message2.blk[64]&maskbits[9])>>9==0?false:true;//2 mod 1
		rf_stat[8] = (message2.blk[64]&maskbits[7])>>7==0?false:true;//1 mod 2
		rf_stat[9] = (message2.blk[64]&maskbits[8])>>8==0?false:true;//2 mod 2
		rf_stat[10] = (message2.blk[64]&maskbits[10])>>10==0?false:true;
		rf_stat[11] = (message2.blk[64]&maskbits[11])>>11==0?false:true;
		rf_stat[12] = (((message2.blk[151])&0x00ff)&maskbits[2])>>2==0?false:true;
		rf_stat[13] = (((message2.blk[151])&0x00ff)&maskbits[3])>>3==0?false:true;
		rf_stat[14] = true;
		rf_stat[15] = true;
		attr.set_value(attr_Rf_stat_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Lock_bo
// 
// description : 	Extract real attribute values for Lock_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Lock_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Lock_bo(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT;j++){
		lock_bo[j] = (message2.blk[53]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Lock_bo_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Lock_inj
// 
// description : 	Extract real attribute values for Lock_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Lock_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Lock_inj(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<4;j++){
		lock_inj[j] = (message2.blk[55]&maskbits[j])>>j==0?false:true;
	}
	lock_inj[4] = ((((message2.blk[32])&0xff00)>>8)&maskbits[5])>>5==0?false:true;
	lock_inj[5] = ((((message2.blk[32])&0xff00)>>8)&maskbits[6])>>6==0?false:true;
	lock_inj[6] = ((((message2.blk[32])&0xff00)>>8)&maskbits[4])>>4==0?false:true;
	for(int j=7;j<NUM_BIT;j++){
		lock_inj[j] = (message2.blk[55]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Lock_inj_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Alarm_door_bo
// 
// description : 	Extract real attribute values for Alarm_door_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Alarm_door_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Alarm_door_bo(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT;j++){
		alarm_door_bo[j] = (message2.blk[44]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Alarm_door_bo_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Alarm_door_inj
// 
// description : 	Extract real attribute values for Alarm_door_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Alarm_door_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Alarm_door_inj(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT;j++){
		alarm_door_inj[j] = (message2.blk[45]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Alarm_door_inj_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Dr_in_inj_stat
// 
// description : 	Extract real attribute values for Dr_in_inj_stat acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Dr_in_inj_stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Dr_in_inj_stat(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT;j++){
		dr_in_inj_stat[j] = (message2.blk[42]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Dr_in_inj_stat_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Dr_in_inj_proc
// 
// description : 	Extract real attribute values for Dr_in_inj_proc acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Dr_in_inj_proc(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Dr_in_inj_proc(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT;j++){
		dr_in_inj_proc[j] = (message2.blk[43]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Dr_in_inj_proc_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Dr_in_bo_proc
// 
// description : 	Extract real attribute values for Dr_in_bo_proc acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Dr_in_bo_proc(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Dr_in_bo_proc(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT;j++){
		dr_in_bo_proc[j] = (message2.blk[41]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Dr_in_bo_proc_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Dr_in_bo_stat
// 
// description : 	Extract real attribute values for Dr_in_bo_stat acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Dr_in_bo_stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Dr_in_bo_stat(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT;j++){
		dr_in_bo_stat[j] = (message2.blk[40]&maskbits[j])>>j==0?false:true;
	}
	attr.set_value(attr_Dr_in_bo_stat_read, NUM_BIT, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Panel_bo
// 
// description : 	Extract real attribute values for Panel_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Panel_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Panel_bo(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<2*NUM_POSTO;j++){
		panelbo[j] = (message2.blk[52]&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Panel_bo_read, 2*NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Panel_inj
// 
// description : 	Extract real attribute values for Panel_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Panel_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Panel_inj(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<2*NUM_POSTO;j++){
		panelinj[j] = (message2.blk[54]&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Panel_inj_read, 2*NUM_POSTO, 0, false);
	
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Badge_ack_bo
// 
// description : 	Extract real attribute values for Badge_ack_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Badge_ack_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Badge_ack_bo(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT+1;j++){
		badgeackbo[j] = ' ';
	}
	bcopy(&(message.badgeackbo[0]), &badgeackbo[0], BADGE_SIZE);
	badgeackbo[NUM_BIT+1] = 0;
	for(int i=0;i<NUM_BIT+1;i++){
		if((badgeackbo[i]<32)||(badgeackbo[i]>126))//filters non printable chars
			badgeackbo[i] = ' ';
	}

	attr.set_value(attr_Badge_ack_bo_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Badge_ack_inj
// 
// description : 	Extract real attribute values for Badge_ack_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Badge_ack_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Badge_ack_inj(Tango::Attribute &attr) entering... "<< endl;
	for(int j=0;j<NUM_BIT+2;j++){
		badgeackinj[j] = ' ';
	}
	bcopy(&(message.badgeackinj[0]), &badgeackinj[0], BADGE_SIZE);
	badgeackinj[NUM_BIT+1] = 0;
	for(int i=0;i<NUM_BIT+1;i++){
		if((badgeackinj[i]<32)||(badgeackinj[i]>126))//filters non printable chars
			badgeackinj[i] = ' ';
	}

	attr.set_value(attr_Badge_ack_inj_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Posto_bo
// 
// description : 	Extract real attribute values for Posto_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Posto_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Posto_bo(Tango::Attribute &attr) entering... "<< endl;
	u_short hmesg = ((message.posti)&0xff00)>>8;
	for(int j=0;j<NUM_POSTO;j++){
		postobo[j] = (hmesg&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Posto_bo_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Badge_bo
// 
// description : 	Extract real attribute values for Badge_bo acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Badge_bo(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Badge_bo(Tango::Attribute &attr) entering... "<< endl;
	
	char badgex[BADGE_SIZE];
	memset(badgex, 0, sizeof(char)*BADGE_SIZE);
	for(int k=0;k<NUM_POSTO;k++){
		for(int j=0;j<NUM_BIT;j++){
			badgebo[k][j] = ' ';
		}
		for(int j=0;j<BADGE_SIZE;j++){
			badgex[j] = message.badge_bo[k][j];
			if((badgex[j]<32)||(badgex[j]>126))
				badgex[j] = ' ';
		}
		strcpy(badgebo[k], badgex);
		badgebo[k][NUM_BIT]=0;
	}
	
	attr.set_value(attr_Badge_bo_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Posto_inj
// 
// description : 	Extract real attribute values for Posto_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Posto_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Posto_inj(Tango::Attribute &attr) entering... "<< endl;
	u_short hmesg = ((message.posti)&0x00ff);
	for(int j=0;j<NUM_POSTO;j++){
		postoinj[j] = (hmesg&maskbits[j])>>j==0?false:true;
	}

	attr.set_value(attr_Posto_inj_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Badge_inj
// 
// description : 	Extract real attribute values for Badge_inj acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Badge_inj(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Badge_inj(Tango::Attribute &attr) entering... "<< endl;
	char badgex[BADGE_SIZE];
	memset(badgex, 0, sizeof(char)*BADGE_SIZE);
	for(int k=0;k<NUM_POSTO;k++){
		for(int j=0;j<NUM_BIT;j++){
			badgeinj[k][j] = ' ';
		}
		for(int j=0;j<BADGE_SIZE;j++){
			badgex[j] = message.badge_inj[k][j];
			if((badgex[j]<32)||(badgex[j]>126))
				badgex[j] = ' ';
		}
		strcpy(badgeinj[k], badgex);
		badgeinj[k][NUM_BIT]=0;
	}

	attr.set_value(attr_Badge_inj_read, NUM_POSTO, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_DBState
// 
// description : 	Extract real attribute values for DBState acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_DBState(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_DBState(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_DBState_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_DBError
// 
// description : 	Extract real attribute values for DBError acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_DBError(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_DBError(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_DBError_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Min_I_TopUp
// 
// description : 	Extract real attribute values for Min_I_TopUp acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Min_I_TopUp(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Min_I_TopUp(Tango::Attribute &attr) entering... "<< endl;
	//attr_Min_I_TopUp_read[0]=srvloop->plc_swap(message2.blk[18]);//TODO: check byte order
	//attr_Min_I_TopUp_read[1]=srvloop->plc_swap(message2.blk[19]);//TODO: check byte order
	attr_Min_I_TopUp_read[0]= message53.sblk[0];
	attr_Min_I_TopUp_read[0]= message53.sblk[1];
	attr.set_value(attr_Min_I_TopUp_read, 2);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Topup_GeV
// 
// description : 	Extract real attribute values for Topup_GeV acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Topup_GeV(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCaccess::read_Topup_GeV(Tango::Attribute &attr) entering... "<< endl;
	//attr_Topup_GeV_read[0]=(message2.blk[27]&maskbits[5])>>5==0?false:true;//moved from DB51.DBX66.5 to DB53.DBX164.0
	//attr_Topup_GeV_read[1]=(message2.blk[27]&maskbits[6])>>6==0?false:true;//moved from DB51.DBX66.6 to DB53.DBX164.1
	attr_Topup_GeV_read[0] = (message53.bblk[2]>>0)&0x1; //bblk 2.0 -> DB53.DBX164.0
	attr_Topup_GeV_read[1] = (message53.bblk[2]>>1)&0x1; //bblk 2.1 -> DB53.DBX164.1
	attr.set_value(attr_Topup_GeV_read, 2);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCaccess::read_Booster_board_disabled_xxx
// 
// description : 	Extract real attribute values for Booster_board_disabled_xxx acquisition result.
//
//-----------------------------------------------------------------------------
void PLCaccess::read_Booster_board_disabled_008(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_008_read[0]=(message2.blk[58]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_008_read[1]=(message2.blk[58]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_008_read[2]=(message2.blk[58]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_008_read[3]=(message2.blk[58]&maskbits[3])>>3==0?false:true;
	attr_Booster_board_disabled_008_read[4]=(message2.blk[58]&maskbits[4])>>4==0?false:true;
	attr_Booster_board_disabled_008_read[5]=(message2.blk[58]&maskbits[10])>>10==0?false:true;
	attr_Booster_board_disabled_008_read[6]=(message2.blk[58]&maskbits[11])>>11==0?false:true;
	attr.set_value(attr_Booster_board_disabled_008_read, 7);
}

void PLCaccess::read_Booster_board_disabled_048(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_048_read[0]=(message2.blk[62]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_048_read[1]=(message2.blk[62]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_048_read[2]=(message2.blk[62]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_048_read[3]=(message2.blk[62]&maskbits[7])>>7==0?false:true;
	attr_Booster_board_disabled_048_read[4]=(message2.blk[62]&maskbits[8])>>8==0?false:true;
	attr_Booster_board_disabled_048_read[5]=(message2.blk[62]&maskbits[10])>>10==0?false:true;
	attr_Booster_board_disabled_048_read[6]=(message2.blk[62]&maskbits[11])>>11==0?false:true;
	attr.set_value(attr_Booster_board_disabled_048_read, 7);
}

void PLCaccess::read_Booster_board_disabled_064(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_064_read[0]=(message2.blk[66]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_064_read[1]=(message2.blk[66]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_064_read[2]=(message2.blk[66]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_064_read[3]=(message2.blk[66]&maskbits[3])>>3==0?false:true;
	attr_Booster_board_disabled_064_read[4]=(message2.blk[66]&maskbits[4])>>4==0?false:true;
	attr_Booster_board_disabled_064_read[5]=(message2.blk[66]&maskbits[5])>>5==0?false:true;
	attr_Booster_board_disabled_064_read[6]=(message2.blk[66]&maskbits[6])>>6==0?false:true;
	attr_Booster_board_disabled_064_read[7]=(message2.blk[66]&maskbits[7])>>7==0?false:true;
	attr_Booster_board_disabled_064_read[8]=(message2.blk[66]&maskbits[8])>>8==0?false:true;
	attr_Booster_board_disabled_064_read[9]=(message2.blk[66]&maskbits[9])>>9==0?false:true;
	attr_Booster_board_disabled_064_read[10]=(message2.blk[66]&maskbits[10])>>10==0?false:true;
	attr_Booster_board_disabled_064_read[11]=(message2.blk[66]&maskbits[11])>>11==0?false:true;
	attr.set_value(attr_Booster_board_disabled_064_read, 12);
}

void PLCaccess::read_Booster_board_disabled_080(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_080_read[0]=(message2.blk[124]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_080_read[1]=(message2.blk[124]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_080_read[2]=(message2.blk[124]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_080_read[3]=(message2.blk[124]&maskbits[3])>>3==0?false:true;
	attr_Booster_board_disabled_080_read[4]=(message2.blk[124]&maskbits[4])>>4==0?false:true;
	attr_Booster_board_disabled_080_read[5]=(message2.blk[124]&maskbits[5])>>5==0?false:true;
	attr_Booster_board_disabled_080_read[6]=(message2.blk[124]&maskbits[6])>>6==0?false:true;
	attr.set_value(attr_Booster_board_disabled_080_read, 7);
}

void PLCaccess::read_Booster_board_disabled_100(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_100_read[0]=(message2.blk[69]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_100_read[1]=(message2.blk[69]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_100_read[2]=(message2.blk[69]&maskbits[2])>>2==0?false:true;
	attr.set_value(attr_Booster_board_disabled_100_read, 3);
}

void PLCaccess::read_Booster_board_disabled_112(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_112_read[0]=(message2.blk[71]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_112_read[1]=(message2.blk[71]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_112_read[2]=(message2.blk[71]&maskbits[2])>>2==0?false:true;
	attr.set_value(attr_Booster_board_disabled_112_read, 3);
}

void PLCaccess::read_Booster_board_disabled_118(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_118_read[0]=(message2.blk[126]&maskbits[0])>>0==0?false:true;
	attr.set_value(attr_Booster_board_disabled_118_read, 1);
}

void PLCaccess::read_Booster_board_disabled_124(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_124_read[0]=(message2.blk[73]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_124_read[1]=(message2.blk[73]&maskbits[1])>>1==0?false:true;
	attr.set_value(attr_Booster_board_disabled_124_read, 2);
}

void PLCaccess::read_Booster_board_disabled_130(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_130_read[0]=(message2.blk[75]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_130_read[1]=(message2.blk[75]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_130_read[2]=(message2.blk[75]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_130_read[3]=(message2.blk[75]&maskbits[3])>>3==0?false:true;
	attr.set_value(attr_Booster_board_disabled_130_read, 4);
}

void PLCaccess::read_Booster_board_disabled_136(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_136_read[0]=(message2.blk[77]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_136_read[1]=(message2.blk[77]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_136_read[2]=(message2.blk[77]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_136_read[3]=(message2.blk[77]&maskbits[3])>>3==0?false:true;
	attr_Booster_board_disabled_136_read[4]=(message2.blk[77]&maskbits[6])>>6==0?false:true;
	attr_Booster_board_disabled_136_read[5]=(message2.blk[77]&maskbits[7])>>7==0?false:true;
	attr.set_value(attr_Booster_board_disabled_136_read, 6);
}

void PLCaccess::read_Booster_board_disabled_142(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_142_read[0]=(((message2.blk[129]&0xff00)>>8)&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_142_read[1]=(((message2.blk[129]&0xff00)>>8)&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_142_read[2]=(((message2.blk[129]&0xff00)>>8)&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_142_read[3]=(((message2.blk[129]&0xff00)>>8)&maskbits[3])>>3==0?false:true;
	attr.set_value(attr_Booster_board_disabled_142_read, 4);
}

void PLCaccess::read_Booster_board_disabled_148(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_148_read[0]=(message2.blk[80]&maskbits[0])>>0==0?false:true;
	attr.set_value(attr_Booster_board_disabled_148_read, 1);
}

void PLCaccess::read_Booster_board_disabled_223(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_223_read[0]=(message2.blk[84]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_223_read[1]=(message2.blk[84]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_223_read[2]=(message2.blk[84]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_223_read[3]=(message2.blk[84]&maskbits[3])>>3==0?false:true;
	attr.set_value(attr_Booster_board_disabled_223_read, 4);
}

void PLCaccess::read_Booster_board_disabled_229(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_229_read[0]=(message2.blk[86]&maskbits[0])>>0==0?false:true;
	attr.set_value(attr_Booster_board_disabled_229_read, 1);
}

void PLCaccess::read_Booster_board_disabled_235(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_235_read[0]=(message2.blk[136]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_235_read[1]=(message2.blk[136]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_235_read[2]=(message2.blk[136]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_235_read[3]=(message2.blk[136]&maskbits[3])>>3==0?false:true;
	attr.set_value(attr_Booster_board_disabled_235_read, 4);
}

void PLCaccess::read_Booster_board_disabled_247(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_247_read[0]=(message2.blk[90]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_247_read[1]=(message2.blk[90]&maskbits[1])>>1==0?false:true;
	attr.set_value(attr_Booster_board_disabled_247_read, 2);
}

void PLCaccess::read_Booster_board_disabled_253(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_253_read[0]=(message2.blk[92]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_253_read[1]=(message2.blk[92]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_253_read[2]=(message2.blk[92]&maskbits[2])>>2==0?false:true;
	attr_Booster_board_disabled_253_read[3]=(message2.blk[92]&maskbits[4])>>4==0?false:true;
	attr_Booster_board_disabled_253_read[4]=(message2.blk[92]&maskbits[5])>>5==0?false:true;
	attr_Booster_board_disabled_253_read[5]=(message2.blk[92]&maskbits[6])>>6==0?false:true;
	attr.set_value(attr_Booster_board_disabled_253_read, 6);
}

void PLCaccess::read_Booster_board_disabled_259(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_259_read[0]=(message2.blk[94]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_259_read[1]=(message2.blk[94]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_259_read[2]=(message2.blk[94]&maskbits[2])>>2==0?false:true;
	attr.set_value(attr_Booster_board_disabled_259_read, 3);
}

void PLCaccess::read_Booster_board_disabled_265(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_265_read[0]=(message2.blk[140]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_265_read[1]=(message2.blk[140]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_265_read[2]=(message2.blk[140]&maskbits[2])>>2==0?false:true;
	attr.set_value(attr_Booster_board_disabled_265_read, 3);
}

void PLCaccess::read_Booster_board_disabled_278(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_278_read[0]=(message2.blk[98]&maskbits[0])>>0==0?false:true;
	attr.set_value(attr_Booster_board_disabled_278_read, 1);
}

void PLCaccess::read_Booster_board_disabled_291(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_291_read[0]=(message2.blk[102]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_291_read[1]=(message2.blk[102]&maskbits[1])>>1==0?false:true;
	attr.set_value(attr_Booster_board_disabled_291_read, 2);
}

void PLCaccess::read_Booster_board_disabled_304(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_304_read[0]=(message2.blk[106]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_304_read[1]=(message2.blk[106]&maskbits[1])>>0==0?false:true;
	attr_Booster_board_disabled_304_read[2]=(message2.blk[106]&maskbits[2])>>0==0?false:true;
	attr.set_value(attr_Booster_board_disabled_304_read, 3);
}

void PLCaccess::read_Booster_board_disabled_310(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_310_read[0]=(message2.blk[108]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_310_read[1]=(message2.blk[108]&maskbits[1])>>1==0?false:true;
	attr.set_value(attr_Booster_board_disabled_310_read, 2);
}

void PLCaccess::read_Booster_board_disabled_316(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_316_read[0]=(((message2.blk[147]&0xff00)>>8)&maskbits[0])>>0==0?false:true; //307.0
	attr_Booster_board_disabled_316_read[1]=(((message2.blk[147]&0xff00)>>8)&maskbits[1])>>1==0?false:true; //307.1
	attr.set_value(attr_Booster_board_disabled_316_read, 2);
}

void PLCaccess::read_Booster_board_disabled_321(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_321_read[0]=(message2.blk[150]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_321_read[1]=(message2.blk[150]&maskbits[1])>>1==0?false:true;
	attr.set_value(attr_Booster_board_disabled_321_read, 2);
}

void PLCaccess::read_Booster_board_disabled_333(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_333_read[0]=(message2.blk[112]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_333_read[1]=(message2.blk[112]&maskbits[1])>>1==0?false:true;
	attr.set_value(attr_Booster_board_disabled_333_read, 2);
}

void PLCaccess::read_Booster_board_disabled_339(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_339_read[0]=(message2.blk[114]&maskbits[0])>>0==0?false:true;
	attr_Booster_board_disabled_339_read[1]=(message2.blk[114]&maskbits[1])>>1==0?false:true;
	attr_Booster_board_disabled_339_read[2]=(message2.blk[114]&maskbits[5])>>5==0?false:true;
	attr.set_value(attr_Booster_board_disabled_339_read, 3);
}

void PLCaccess::read_Booster_board_disabled_345(Tango::Attribute &attr)
{
	attr_Booster_board_disabled_345_read[0]=(message2.blk[154]&maskbits[0])>>0==0?false:true;
	attr.set_value(attr_Booster_board_disabled_345_read, 1);
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::open_bst_b13_1
 *
 *	description:	method to execute "Open_bst_b13_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::open_bst_b13_1()
{
	//DEBUG_STREAM << "PLCaccess::open_bst_b13_1(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_B13_1 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.OPN_CMD_BST_B13_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::close_bst_b13_1
 *
 *	description:	method to execute "Close_bst_b13_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::close_bst_b13_1()
{
	//DEBUG_STREAM << "PLCaccess::close_bst_b13_1(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_B13_1 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.CLD_CMD_BST_B13_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::open_bst_bts1_1
 *
 *	description:	method to execute "Open_bst_bts1_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::open_bst_bts1_1()
{
	//DEBUG_STREAM << "PLCaccess::open_bst_bts1_1(): entering... !" << endl;

	//	Add your own code to control device here
	
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_BTS1_1 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.OPN_CMD_BST_BTS1_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::close_bst_bts1_1
 *
 *	description:	method to execute "Close_bst_bts1_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::close_bst_bts1_1()
{
	//DEBUG_STREAM << "PLCaccess::close_bst_bts1_1(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_BTS1_1 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.CLD_CMD_BST_BTS1_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::open_bst_bts1_2
 *
 *	description:	method to execute "Open_bst_bts1_2"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::open_bst_bts1_2()
{
	//DEBUG_STREAM << "PLCaccess::open_bst_bts1_2(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_BTS1_2 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.OPN_CMD_BST_BTS1_2 = CMD_OK;
	send_cmd();
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::close_bst_bts1_2
 *
 *	description:	method to execute "Close_bst_bts1_2"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::close_bst_bts1_2()
{
	//DEBUG_STREAM << "PLCaccess::close_bst_bts1_2(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_BTS1_2 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.CLD_CMD_BST_BTS1_2 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::open_bst_bts1_3
 *
 *	description:	method to execute "Open_bst_bts1_3"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::open_bst_bts1_3()
{
	//DEBUG_STREAM << "PLCaccess::open_bst_bts1_3(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_BTS1_3 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.OPN_CMD_BST_BTS1_3 = CMD_OK;
	send_cmd();
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::close_bst_bts1_3
 *
 *	description:	method to execute "Close_bst_bts1_3"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::close_bst_bts1_3()
{
	//DEBUG_STREAM << "PLCaccess::close_bst_bts1_3(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_BTS1_3 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.CLD_CMD_BST_BTS1_3 = CMD_OK;
	send_cmd();
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::reint_pasv
 *
 *	description:	method to execute "Reint_pasv"
 *	command for activating PLC boards after an error or a breakdown
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::reint_pasv()
{
	//DEBUG_STREAM << "PLCaccess::reint_pasv(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.REINT_PASS1 = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.REINT_PASS1 = CMD_OK;
	send_cmd();
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::close_bst_bl
 *
 *	description:	method to execute "Close_bst_bl"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::close_bst_bl()
{
	//DEBUG_STREAM << "PLCaccess::close_bst_bl(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.FRC_CLD_BST_BL = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.FRC_CLD_BST_BL = CMD_OK;
	send_cmd();
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::set_top_up
 *
 *	description:	method to execute "Set_top_up"
 *	Switches to TOP-UP mode
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::set_top_up()
{
	//DEBUG_STREAM << "PLCaccess::set_top_up(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.SET_TOP_UP = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.SET_TOP_UP = CMD_OK;
	send_cmd();
	cmd_on_topup = true;
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::ack_eme
 *
 *	description:	method to execute "Ack_eme"
 *	Acknowledge for emergency button pressed
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::ack_eme()
{
	//DEBUG_STREAM << "PLCaccess::ack_eme(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.ACK_EME = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.ACK_EME = CMD_OK;
	send_cmd();
	
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::call_ronda_bo
 *
 *	description:	method to execute "Call_ronda_bo"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::call_ronda_bo()
{
	//DEBUG_STREAM << "PLCaccess::call_ronda_bo(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CALL_RDA_BO;
	send_cmd();
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::start_ronda_bo
 *
 *	description:	method to execute "Start_ronda_bo"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::start_ronda_bo()
{
	//DEBUG_STREAM << "PLCaccess::start_ronda_bo(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_STRT_RDA_BO;
	send_cmd();

	doors_ronda_all[3] = true; //when ronda is ok, closed doors become white
	doors_ronda_all[8] = true; 
	//reset of all stored bits
	for(int j=0;j<4;j++){
		memo_barr[j] = false;
	}
	ronda_bo_fail_cause[0] = false;
	ronda_bo_fail_cause[1] = false;
	ronda_bo_fail_cause[12] = false;
	ronda_bo_fail_cause[13] = false;
	ronda_bo_fail_cause[14] = false;
	ronda_bo_fail_cause[15] = false;
	memo_err_inout[0] = false;
	memo_err_inout[1] = false;


}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::end_ronda_bo
 *
 *	description:	method to execute "End_ronda_bo"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::end_ronda_bo()
{
	//DEBUG_STREAM << "PLCaccess::end_ronda_bo(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_END_RDA_BO;
	send_cmd();


}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::start_ronda_int
 *
 *	description:	method to execute "Start_ronda_int"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::start_ronda_int()
{
	//DEBUG_STREAM << "PLCaccess::start_ronda_int(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_STRT_RDA_INT;
	send_cmd();

	doors_ronda_all[16] = true; 
	doors_ronda_all[17] = true; 
	doors_ronda_all[18] = true; 

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::end_ronda_int
 *
 *	description:	method to execute "End_ronda_int"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::end_ronda_int()
{
	//DEBUG_STREAM << "PLCaccess::end_ronda_int(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_END_RDA_INT;
	send_cmd();


}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::start_ronda_dx
 *
 *	description:	method to execute "Start_ronda_dx"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::start_ronda_dx()
{
	//DEBUG_STREAM << "PLCaccess::start_ronda_dx(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_STRT_RDA_DX;
	send_cmd();

	doors_ronda_all[10] = true; 
	doors_ronda_all[13] = true; 
	doors_ronda_all[14] = true; 
	doors_ronda_all[15] = true; 

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::end_ronda_dx
 *
 *	description:	method to execute "End_ronda_dx"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::end_ronda_dx()
{
	//DEBUG_STREAM << "PLCaccess::end_ronda_dx(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_END_RDA_DX;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::start_ronda_sx
 *
 *	description:	method to execute "Start_ronda_sx"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::start_ronda_sx()
{
	//DEBUG_STREAM << "PLCaccess::start_ronda_sx(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_STRT_RDA_SX;
	send_cmd();

	doors_ronda_all[9] = true; 
	doors_ronda_all[4] = true; 
	doors_ronda_all[5] = true; 
	doors_ronda_all[7] = true; 
	doors_ronda_all[6] = true; 
	doors_ronda_all[12] = true; 
	doors_ronda_all[11] = true; 

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::end_ronda_sx
 *
 *	description:	method to execute "End_ronda_sx"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::end_ronda_sx()
{
	//DEBUG_STREAM << "PLCaccess::end_ronda_sx(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_END_RDA_SX;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::start_ronda_roof
 *
 *	description:	method to execute "Start_ronda_roof"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::start_ronda_roof()
{
	//DEBUG_STREAM << "PLCaccess::start_ronda_roof(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_STRT_RDA_ROOF;
	send_cmd();
	doors_ronda_all[19] = true; 
	doors_ronda_all[15] = true; 

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::end_ronda_roof
 *
 *	description:	method to execute "End_ronda_roof"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::end_ronda_roof()
{
	//DEBUG_STREAM << "PLCaccess::end_ronda_roof(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_END_RDA_ROOF;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::ripr_in_boost
 *
 *	description:	method to execute "Ripr_in_boost"
 *	Reset inputs of booster door after access failed
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::ripr_in_boost()
{
	//DEBUG_STREAM << "PLCaccess::ripr_in_boost(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=RIPR_ING_BO;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::ripr_in_inj
 *
 *	description:	method to execute "Ripr_in_inj"
 *	Reset inputs of injector door after access failed
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::ripr_in_inj()
{
	//DEBUG_STREAM << "PLCaccess::ripr_in_inj(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=RIPR_ING_INJ;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::open_door_bo
 *
 *	description:	method to execute "Open_door_bo"
 *	Open booster's door
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::open_door_bo()
{
	//DEBUG_STREAM << "PLCaccess::open_door_bo(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_OPN_DR_BO;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::open_door_inj
 *
 *	description:	method to execute "Open_door_inj"
 *	Open injector's door
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::open_door_inj()
{
	//DEBUG_STREAM << "PLCaccess::open_door_inj(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_OPN_DR_INJ;
	send_cmd();

}


//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::send_frame()
 *
 * Description:	Send a  frame to a node via  TCP/IP
 *
 * Arg(s) In:	unsigned char *query -  data to send
 *		short query_length - number of data words to send
 *
 *
 */
//+------------------------------------------------------------------

void PLCaccess::send_frame (unsigned char *query, short query_length)
{
	unsigned char frame[BUF_LEN];
   	memset ( frame, 0, sizeof(frame) );
	unsigned short int sbuf[BUF_LEN];
        memset (sbuf,0,1+(2*query_length));
	bcopy(&query[0], &sbuf, 2*query_length);
	for (int i=0; i<query_length; i++){
		sbuf[i] = htons(sbuf[i]);
	}
	bcopy(&sbuf, &frame[0], 2*query_length);
			
   	int writecnt = write (new_sd, frame,query_length);
	if(writecnt ==-1){
		set_state(Tango::UNKNOWN);
		set_status("error writing socket");
		DEBUG_STREAM << "PLCaccess::send_frame(): error writing socket !" << endl;
	}
	else{
   		set_state(Tango::ON);
		set_status("mesg sent");
		DEBUG_STREAM << "PLCaccess::send_frame(): written " << writecnt << "chars" <<endl;
	}
	
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::send_cmd
 *
 *	description:	method to execute "Send_cmd"
 *
 * @param	argin	hex string representing single acknowledges
 * @return	command result
 *
 */
//+------------------------------------------------------------------
void PLCaccess::send_cmd(void)
{
	DEBUG_STREAM << "PLCsocket::send_cmd(): entering... !" << endl;

	//	Add your own code to control device here

	unsigned char query[BUF_LEN];
        memset (query,0,BUF_LEN);
	try{
	//write the message to PLC 
	
		bcopy(&cmd_msg, &query[0], sizeof(plc_tx));		
		send_frame(query, sizeof(plc_tx));
	
	}
	catch(Tango::DevFailed& e) {
		set_state(Tango::FAULT);
		set_status("Write command failed");
		Tango::Except::re_throw_exception(e,
			(const char *) "write error",
			(const char *) "write cmd error",
			(const char *) "PLCaccess::send_cmd", Tango::ERR);					
	}
	

	return;
}


//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::del_posto_bo
 *
 *	description:	method to execute "Del_posto_bo"
 *	Deletes incorrect access of booster door corresponding to argin place (from 0 to 7)
 *
 * @param	argin	places of name to be deleted
 *
 */
//+------------------------------------------------------------------
void PLCaccess::del_posto_bo(Tango::DevUShort argin)
{
	DEBUG_STREAM << "PLCaccess::del_posto_bo(): entering... !" << endl;

	//	Add your own code to control device here
	int num = argin;
	if(num<0||num>7){
		TangoSys_MemStream os;
		os << "posto must be a number between 0 and 7" << ends;
		Tango::Except::throw_exception(
			(const char *)"wrong argument",
			os.str(),
			(const char *)"PLCaccess::del_posto_bo()");
	}
	else{
	
		memset(&cmd_msg, 0, sizeof(plc_tx));
		cmd_msg.nDB = DEL_BADGE_CMD;
		//cmd_msg.posti_occupati = (((message.posti>>8)&(posti[num]))<<8)|(message.posti&0x00ff);
		//send only 1 bit as 0, 15 bit as 1
		cmd_msg.posti_occupati = (posti[num]<<8) | 0x00ff;
		send_cmd();
		DEBUG_STREAM << "PLCaccess::del_name(): badge deleted" << endl;
	}

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::del_posto_inj
 *
 *	description:	method to execute "Del_posto_inj"
 *	Deletes incorrect access of injector door corresponding to argin place (from 0 to 7)
 *
 * @param	argin	place of name to be deleted
 *
 */
//+------------------------------------------------------------------
void PLCaccess::del_posto_inj(Tango::DevUShort argin)
{
	DEBUG_STREAM << "PLCaccess::del_posto_inj(): entering... !" << endl;

	//	Add your own code to control device here
	int num = argin;
	if(num<0||num>15){
		TangoSys_MemStream os;
		os << "posto must be a number between 8 and 15" << ends;
		Tango::Except::throw_exception(
			(const char *)"wrong argument",
			os.str(),
			(const char *)"PLCaccess::del_posto_inj()");
	}
	else {
		if(num<16&&num>7)
			num-=8;
		memset(&cmd_msg, 0, sizeof(plc_tx));
		cmd_msg.nDB = DEL_BADGE_CMD;
		//cmd_msg.posti_occupati = message.posti&((posti[num]));
		//send only 1 bit as 0, 15 bit as 1
		cmd_msg.posti_occupati = posti[num];
		send_cmd();
		DEBUG_STREAM << "PLCaccess::del_name(): badge deleted" << endl;
	}

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_open_13_1
 *
 *	description:	method to execute "Confirm_open_13_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_open_13_1()
{
	DEBUG_STREAM << "PLCaccess::confirm_open_13_1(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_B13_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_close_13_1
 *
 *	description:	method to execute "Confirm_close_13_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_close_13_1()
{
	DEBUG_STREAM << "PLCaccess::confirm_close_13_1(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_B13_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_open_1_1
 *
 *	description:	method to execute "Confirm_open_1_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_open_1_1()
{
	DEBUG_STREAM << "PLCaccess::confirm_open_1_1(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_BTS1_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_close_1_1
 *
 *	description:	method to execute "Confirm_close_1_1"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_close_1_1()
{
	DEBUG_STREAM << "PLCaccess::confirm_close_1_1(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_BTS1_1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_open_1_2
 *
 *	description:	method to execute "Confirm_open_1_2"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_open_1_2()
{
	DEBUG_STREAM << "PLCaccess::confirm_open_1_2(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_BTS1_2 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_close_1_2
 *
 *	description:	method to execute "Confirm_close_1_2"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_close_1_2()
{
	DEBUG_STREAM << "PLCaccess::confirm_close_1_2(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_BTS1_2 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_open_1_3
 *
 *	description:	method to execute "Confirm_open_1_3"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_open_1_3()
{
	DEBUG_STREAM << "PLCaccess::confirm_open_1_3(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.OPN_CMD_BST_BTS1_3 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_close_1_3
 *
 *	description:	method to execute "Confirm_close_1_3"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_close_1_3()
{
	DEBUG_STREAM << "PLCaccess::confirm_close_1_3(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.CLD_CMD_BST_BTS1_3 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_reint_pasv
 *
 *	description:	method to execute "Confirm_reint_pasv"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_reint_pasv()
{
	DEBUG_STREAM << "PLCaccess::confirm_reint_pasv(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.REINT_PASS1 = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_close_bst_bl
 *
 *	description:	method to execute "Confirm_close_bst_bl"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_close_bst_bl()
{
	DEBUG_STREAM << "PLCaccess::confirm_close_bst_bl(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.FRC_CLD_BST_BL = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_set_topup
 *
 *	description:	method to execute "Confirm_set_topup"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_set_topup()
{
	DEBUG_STREAM << "PLCaccess::confirm_set_topup(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.SET_TOP_UP = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::confirm_ack_eme
 *
 *	description:	method to execute "Confirm_ack_eme"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::confirm_ack_eme()
{
	DEBUG_STREAM << "PLCaccess::confirm_ack_eme(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.ACK_EME = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::echo_db50
 *
 *	description:	method to execute "Echo_db50"
 *	request from plc a copy of db50, the result is mapped on the attributes
 *
 * @return	success of the request
 *
 */
//+------------------------------------------------------------------
Tango::DevString PLCaccess::echo_db50()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevString	argout  = new char[6];
	strcpy(argout, " ");
	DEBUG_STREAM << "PLCaccess::echo_db50(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_50_CMD;
	send_cmd();

	echoreq = 50;
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::echo_db51
 *
 *	description:	method to execute "Echo_db51"
 *	requests to plc a copy of db51, the result is stored inside the attributes
 *
 * @return	success of request
 *
 */
//+------------------------------------------------------------------
Tango::DevString PLCaccess::echo_db51()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevString	argout  = new char[6];
	strcpy(argout, " ");
	DEBUG_STREAM << "PLCaccess::echo_db51(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_51_CMD;
	send_cmd();

	echoreq = 51;
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::echo_db121
 *
 *	description:	method to execute "Echo_db121"
 *	Request the list of ronda enabled people names
 *
 * @return	result
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *PLCaccess::echo_db121()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();
	DEBUG_STREAM << "PLCaccess::echo_db121(): entering... !" << endl;

	//	Add your own code to control device here

	argout->length(DB_NAMES_LEN);
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_121_CMD;
	send_cmd();
	echoreq = 121;
	usleep(TIME_CONFIRM);
	pers_name pn;
	
	for(int k=0;k<DB_NAMES_LEN;k++){
		for(int j=0;j<NUM_BIT;j++){
			pn[j] = (unsigned char)(ronda_names.pn[k])[j];
		}
		(*argout)[k] = CORBA::string_dup((const char*)pn);
		
	}
	return argout;

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::echo_db122
 *
 *	description:	method to execute "Echo_db122"
 *	Request the list of controlled access people names
 *
 * @return	result
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *PLCaccess::echo_db122()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();
	DEBUG_STREAM << "PLCaccess::echo_db122(): entering... !" << endl;

	//	Add your own code to control device here

	argout->length(DB_NAMES_LEN);
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = ECHO_122_CMD;
	send_cmd();
	echoreq = 122;

	usleep(TIME_CONFIRM);
	pers_name pn;
	for(int k=0;k<DB_NAMES_LEN;k++){
		for(int j=0;j<NUM_BIT;j++){
			pn[j] = (unsigned char)(ctrl_names.pn[k])[j];
		}
		(*argout)[k] = CORBA::string_dup((const char*)pn);
		
	}
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::log_to_db
 *
 *	description:	method to execute the logging to a textfile
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
/*void PLCaccess::log_to_db(db_mesg ms, tstamp_t t)
{
	
}
*/

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::tofile
 *
 *	description:	method to execute the logging to a textfile
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
/*void PLCaccess::tofile(int btype, int len)
{
	DEBUG_STREAM << "PLCaccess::tofile(): entering... !" << endl;
	char buf[BUF_WR];//buffer for single digit conversion
   	int writefile = 0;
	string x;
	
	if(btype==51){
		x.append("DB51-");
   		memset (buf, 0, BUF_WR);
		sprintf(buf, "len:%d\t", len);
		x.append(buf);
   		memset (buf, 0, BUF_WR);
		sprintf(buf, "%02x/%02x/%02x %02x:%02x:%02x.%03x\t",message2.day,message2.month,message2.year,message2.hour,message2.min, message2.sec,message2.msec);
		x.append(buf);
		for(int j=0;j<157;j++){
   			memset (buf, 0, BUF_WR);
			sprintf(buf, "%d-%04x/",j,message2.blk[j]);
			x.append(buf);
		}
   		memset (buf, 0, BUF_WR );
		sprintf(buf, "\test_bo:%ld",message2.msg_est_bo);
		x.append(buf);
   		memset (buf, 0, BUF_WR );
		sprintf(buf, "\tint_bo:%ld",message2.msg_int_bo);
		x.append(buf);
   		memset (buf, 0, BUF_WR );
		sprintf(buf, "\test_inj:%ld",message2.msg_est_inj);
		x.append(buf);
   		memset (buf, 0, BUF_WR );
		sprintf(buf, "\tint_inj:%ld",message2.msg_int_inj);
		x.append(buf);
	}
	if(btype==50){
 			x.append("DB50-");
   			memset (buf, 0, BUF_WR );
			sprintf(buf, "len:%d\t", len);
			x.append(buf);
   		memset (buf, 0, BUF_WR);
		sprintf(buf, "%02x/%02x/%02x %02x:%02x:%02x\t",message.day,message.month,message.year,message.hour,message.min, message.sec);
		x.append(buf);
   		memset (buf, 0, BUF_WR );
		sprintf(buf, "posti:%04x\t", message.posti);
		x.append(buf);
		for(int j=0;j<8;j++){
   			memset (buf, 0, BUF_WR );
			snprintf(buf, 17, "B%d-%s/",j,message.badge_bo[j]);
			x.append(buf);
		}
		for(int j=0;j<8;j++){
   			memset (buf, 0, BUF_WR);
			snprintf(buf, 17, "I%d-%s/",j,message.badge_inj[j]);
			x.append(buf);
		}
   		memset (buf, 0, BUF_WR );
		sprintf(buf, "\tbadgeackbo:%s\t",message.badgeackbo);
		x.append(buf);
   		memset (buf, 0, BUF_WR );
		sprintf(buf, "badgeackinj:%s\t",message.badgeackinj);
		x.append(buf);
	}
	if(btype==121){
 			x.append("DB121-");
   			memset (buf, 0, BUF_WR );
			sprintf(buf, "len:%d\t", len);
			x.append(buf);
			for(int j=0;j<116;j++){
   				memset (buf, 0, BUF_WR );
				sprintf(buf, "%d-%s/",j,ronda_names.pn[j]);
				x.append(buf);
			}
	}
	if(btype==122){
 			x.append("DB122-");
   			memset (buf, 0, BUF_WR );
			sprintf(buf, "len:%d\t", len);
			x.append(buf);
			for(int j=0;j<116;j++){
   				memset (buf, 0, BUF_WR );
				sprintf(buf, "%d-%s/",j,ctrl_names.pn[j]);
				x.append(buf);
			}
	}
	if(btype==0){//undefined
 			x.append("undefined type");
	}
	x.append("\n");
	
	if((writefile = write (fdlog, x.c_str(), x.length()))<0){
		if(get_state()!=Tango::FAULT)
			set_state(Tango::DISABLE);
		set_status("File logging failure");
		DEBUG_STREAM << "PLCaccess::log_it(): File logging failure" << endl;
   	}
	else{
		if(get_state()!=Tango::FAULT)
			set_state(Tango::ON);
		set_status("Log written");
	}
	DEBUG_STREAM << "PLCaccess::log_it(): finished" << endl;
	
}*/

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::gun_on
 *
 *	description:	method to execute "Gun_on"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::gun_on()
{
	DEBUG_STREAM << "PLCaccess::gun_on(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.GUN_ON_CMD = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.GUN_ON_CMD = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::gun_off
 *
 *	description:	method to execute "Gun_off"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::gun_off()
{
	DEBUG_STREAM << "PLCaccess::gun_off(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.GUN_OFF_CMD = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.GUN_OFF_CMD = CMD_OK;
	send_cmd();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::syncronize
 *
 *	description:	method to execute "Syncronize"
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::syncronize()
{
	DEBUG_STREAM << "PLCaccess::syncronize(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0x0000, sizeof(plc_tx));
	cmd_msg.nDB = SYNC_CMD;
	timeval seco;
	gettimeofday(&seco,NULL);
        time_t tim=time(NULL);
        tm *now=localtime(&tim);
	char buf[10];
	
	uint8_t mon = now->tm_mon+1;
	uint8_t year = now->tm_year-100;
	uint8_t hour = now->tm_hour;
	uint8_t day = now->tm_mday;
	uint8_t sec = now->tm_sec;
	uint8_t min = now->tm_min;
	uint8_t wday = now->tm_wday+1;
	uint8_t msec = 0;
	bcopy(&mon, &(cmd_msg.month), 1);
	bcopy(&year, &(cmd_msg.year), 1);
	bcopy(&hour, &(cmd_msg.hour), 1);
	bcopy(&day, &(cmd_msg.day), 1);
	bcopy(&sec, &(cmd_msg.sec), 1);
	bcopy(&min, &(cmd_msg.min), 1);
	bcopy(&wday, &(cmd_msg.wday), 1);
	bcopy(&msec, &(cmd_msg.msec), 1);

	sprintf(buf, "%02d", (now->tm_year-100));
	cmd_msg.year = ((buf[0]-'0')<<4)+(buf[1]-'0');
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_mon+1));
	cmd_msg.month = ((buf[0]-'0')<<4)+(buf[1]-'0');
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_mday));
	cmd_msg.day = ((buf[0]-'0')<<4)+(buf[1]-'0');
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_hour));
	cmd_msg.hour = ((buf[0]-'0')<<4)+(buf[1]-'0');
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_min));
	cmd_msg.min = ((buf[0]-'0')<<4)+(buf[1]-'0');
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_sec));
	cmd_msg.sec = ((buf[0]-'0')<<4)+(buf[1]-'0');
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%01d", (now->tm_wday+1));
	cmd_msg.wday = ((0<<4)+(buf[0]-'0'))&0x00ffff;
	cmd_msg.msec = 0;

	DEBUG_STREAM <<"setting PLC time to:"<< endl;
	DEBUG_STREAM <<"year:"<<year <<" month:"<<mon <<" day:"<<day <<" hour:"<<hour <<" min:"<< min<<" sec:"<<sec<< endl;
	DEBUG_STREAM <<"mesg len:"<<sizeof(plc_tx) << endl;

	send_cmd();
}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::rst_top_up
 *
 *	description:	method to execute "Rst_top_up"
 *	Reset TOP-UP mode
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::rst_top_up()
{
	DEBUG_STREAM << "PLCaccess::rst_top_up(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = BST_CMD;
	cmd_msg.RSET_TOP_UP = CMD_REQ;
	send_cmd();
	
	usleep(TIME_CONFIRM);
	cmd_msg.RSET_TOP_UP = CMD_OK;
	send_cmd();
	cmd_rst_err = true;

}

//+------------------------------------------------------------------
/**
 *	method:	PLCaccess::err_topup_rst
 *
 *	description:	method to execute "Err_topup_rst"
 *	Reset topup errors
 *
 *
 */
//+------------------------------------------------------------------
void PLCaccess::err_topup_rst()
{
	DEBUG_STREAM << "PLCaccess::err_topup_rst(): entering... !" << endl;

	//	Add your own code to control device here

	memset(&cmd_msg, 0, sizeof(plc_tx));
	cmd_msg.nDB = RONDA_CMD;
	cmd_msg.ronda_stat=CMD_RST_TOPUP_ERR;
	send_cmd();
	cmd_rst_err = false;
}

time_t PLCaccess::calc_timeval(uint8_t year,uint8_t month,uint8_t day,uint8_t hour,uint8_t min,uint8_t sec,uint8_t wday)
{
	char buffer [10];
	tm t2;
	time_t tn;

	memset (buffer,0,10);
	sprintf (buffer, "%x", sec);
	t2.tm_sec = atoi(buffer);
	memset (buffer,0,10);
	sprintf (buffer, "%x", min);
	t2.tm_min = atoi(buffer);
	memset (buffer,0,10);
	sprintf (buffer, "%x", hour);
	t2.tm_hour = atoi(buffer);
	memset (buffer,0,10);
	sprintf (buffer, "%x", day);
	t2.tm_mday = atoi(buffer);
	memset (buffer,0,10);
	sprintf (buffer, "%x", month);
	t2.tm_mon = atoi(buffer)-1;
	memset (buffer,0,10);
	sprintf (buffer, "%x", year);
	t2.tm_year = atoi(buffer)+100;
	memset (buffer,0,10);
	sprintf (buffer, "%x", wday);
	t2.tm_wday = atoi(buffer)-1;
	
	//0: consider time to convert never dst, time converted automatically adjusted with dst
	//-1: consider time to convert already ok with dst, so no adjustment applied by mktime
	t2.tm_isdst = 0;//*-1
	
	DEBUG_STREAM << __func__<<": year=0x"<<hex<<(int)year<<" month=0x"<<(int)month<<" day=0x"<<(int)day<<" hour=0x"<<(int)hour<<" min=0x"<<(int)min<<" sec=0x"<<(int)sec<<dec<<endl;
	tn = mktime(&t2);//-timezone;//with -timezone as if GMT
	DEBUG_STREAM << __func__<<": --> tn="<<tn<<" timezone="<<timezone<<" t2.tm_isdst="<<(int)t2.tm_isdst<<endl;
	if(t2.tm_isdst == 1)
	{
		DEBUG_STREAM << __func__<<": --> ORA SOLARE! corrected to hour="<<(int)t2.tm_hour<<" min="<<(int)t2.tm_min<<" sec="<<(int)t2.tm_sec<<endl;
	}
	return tn;
}	



int32_t PLCaccess::calc_time_ms(uint8_t msec,uint8_t wday)
{
	char buffer [10];
	int32_t d = 0;
	int32_t ms_new;
	int c1, c2;
	ms_new = 0;
	memset (buffer,0,10);
	sprintf (buffer, "%x", msec);
	c1 = atoi(buffer);
	memset (buffer,0,10);
	wday = wday >> 4;
	sprintf (buffer, "%x", wday);
	c2 = atoi(buffer);
	ms_new = c1*10+c2;
	d = ms_new;
	
	return d;

}

}	//	namespace
